
import collections
from itertools import dropwhile
import logging

import simuvex

from .errors import AngrError

l = logging.getLogger('angr.call_stack')

class CallFrame(object):
    """
    Stores the address of the function you're in and the value of SP
    at the VERY BOTTOM of the stack, i.e. points to the return address.
    """
    def __init__(self, state=None, call_site_addr=None, func_addr=None, stack_ptr=None, ret_addr=None, jumpkind=None):
        """
        Initialize with either a state or the function address,
        stack pointer, and return address
        """

        self.jumpkind = jumpkind if jumpkind is not None else (state.scratch.jumpkind if state is not None else None)
        self.call_site_addr = call_site_addr

        if state is not None:
            try:
                self.func_addr = state.se.any_int(state.ip)
                self.stack_ptr = state.se.any_int(state.regs.sp)
            except (simuvex.SimUnsatError, simuvex.SimSolverModeError, AttributeError):
                self.func_addr = None
                self.stack_ptr = None

            if self.jumpkind and self.jumpkind.startswith('Ijk_Sys'):
                # syscalls
                self.ret_addr = state.regs.ip_at_syscall
            else:
                # calls
                if state.arch.call_pushes_ret:
                    self.ret_addr = state.memory.load(state.regs.sp, state.arch.bits / 8,
                                                      endness=state.arch.memory_endness, inspect=False
                                                      )
                else:
                    self.ret_addr = state.regs.lr

            # Try to convert the ret_addr to an integer
            try:
                if self.ret_addr.symbolic:
                    l.debug('CallStack does not support symbolic return addresses for performance concerns.')
                    self.ret_addr = None
                else:
                    self.ret_addr = state.se.any_int(self.ret_addr)
            except (simuvex.SimUnsatError, simuvex.SimSolverModeError):
                self.ret_addr = None
        else:
            self.func_addr = func_addr
            self.stack_ptr = stack_ptr
            self.ret_addr = ret_addr

        self.block_counter = collections.Counter()

    def __str__(self):
        return "Func %#x, sp=%#x, ret=%#x" % (self.func_addr, self.stack_ptr, self.ret_addr)

    def __repr__(self):
        return '<CallFrame (Func %#x)>' % self.func_addr

    #
    # Properties
    #

    @property
    def call_target(self):
        return self.func_addr

    @property
    def return_target(self):
        return self.ret_addr

    @property
    def stack_pointer(self):
        return self.stack_ptr

    #
    # Public methods
    #

    def copy(self):
        c = CallFrame(state=None, call_site_addr=self.call_site_addr, func_addr=self.func_addr,
                      stack_ptr=self.stack_ptr, ret_addr=self.ret_addr, jumpkind=self.jumpkind
                      )
        c.block_counter = collections.Counter(self.block_counter)
        return c


class CallStackAction(object):
    """
    Used in callstack backtrace, which is a history of callstacks along a path, to record individual actions occurred
    each time the callstack is changed.
    """
    def __init__(self, callstack_hash, callstack_depth, action, callframe=None, ret_site_addr=None):
        self.callstack_hash = callstack_hash
        self.callstack_depth = callstack_depth
        self.action = action

        if action not in ('push', 'pop'):
            raise AngrError('Unsupported action string "%s".' % action)

        self.callframe = callframe
        self.ret_site_addr = ret_site_addr

        if action == 'push' and self.callframe is None:
            raise AngrError('callframe must be specified when action is "push".')

        elif action == 'pop' and self.callframe is not None:
            raise AngrError('callframe must not be specified when action is "pop".')

    def __repr__(self):
        if self.action == 'push':
            return "<CallStackAction push with %s>" % self.callframe
        else: # pop
            return "<CallStackAction pop, ret site %#x>" % self.ret_site_addr


class CallStack(object):
    """
    Represents a call stack.
    """
    def __init__(self, stack=None):

        if stack is None:
            self._callstack = [ ]
        else:
            self._callstack = stack

    #
    # Overriden methods
    #

    def __iter__(self):
        """
        Iterate through the callstack, from top to bottom
        (most recent first).
        """
        for cf in reversed(self._callstack):
            yield cf

    def __getitem__(self, k):
        """
        Returns the CallFrame at index k, indexing from the top of the stack.
        """
        k = -1 - k
        return self._callstack[k]

    def __len__(self):
        """
        Get how many frames there are in the current call stack.

        :return: Number of frames
        :rtype: int
        """

        return len(self._callstack)

    def __repr__(self):
        """
        Get a string representation.

        :return: A printable representation of the CallStack object
        :rtype: str
        """
        return "<CallStack (depth %d)>" % len(self._callstack)

    def __str__(self):
        return "Backtrace:\n%s" % "\n".join(str(f) for f in self)

    def __eq__(self, other):
        if not isinstance(other, CallStack):
            return False

        if len(self) != len(other):
            return False

        for c1, c2 in zip(self._callstack, other._callstack):
            if c1.func_addr != c2.func_addr or c1.stack_ptr != c2.stack_ptr or c1.ret_addr != c2.ret_addr:
                return False

        return True

    def __ne__(self, other):
        return self != other

    def __hash__(self):
        return hash(tuple((c.func_addr, c.stack_ptr, c.ret_addr) for c in self._callstack))

    #
    # Properties
    #

    @property
    def current_function_address(self):
        """
        Address of the current function.

        :return: the address of the function
        :rtype: int
        """

        if len(self._callstack) == 0:
            return 0  # This is the root level
        else:
            frame = self._callstack[-1]
            return frame.func_addr

    @current_function_address.setter
    def current_function_address(self, func_addr):
        """
        Set the address of the current function. Note that we must make a copy of the CallStackFrame as CallStackFrame
        is considered to be immutable.

        :param int func_addr: The function address.
        :return: None
        """

        frame = self._callstack[-1].copy()
        frame.func_addr = func_addr
        self._callstack[-1] = frame

    @property
    def current_stack_pointer(self):
        """
        Get the value of the stack pointer.

        :return: Value of the stack pointer
        :rtype: int
        """
        if len(self._callstack) == 0:
            return None
        else:
            frame = self._callstack[-1]
            return frame.stack_pointer

    @property
    def current_return_target(self):
        """
        Get the return target.

        :return: The address of return target.
        :rtype: int
        """

        if len(self._callstack) == 0:
            return None
        return self._callstack[-1].return_target

    #
    # Static methods
    #

    @staticmethod
    def stack_suffix_to_string(stack_suffix):
        """
        Convert a stack suffix to a human-readable string representation.
        :param tuple stack_suffix: The stack suffix.
        :return: A string representation
        :rtype: str
        """
        s = "[" + ",".join([("0x%x" % i) if i is not None else "Unspecified" for i in stack_suffix]) + "]"
        return s

    @staticmethod
    def _rfind(lst, item):
        """
        Reverse look-up.

        :param list lst: The list to look up in.
        :param item: The item to look for.
        :return: Offset of the item if found. A ValueError is raised if the item is not in the list.
        :rtype: int
        """

        try:
            return dropwhile(lambda x: lst[x] != item,
                             reversed(xrange(len(lst)))).next()
        except Exception:
            raise ValueError("%s not in the list" % item)

    @property
    def top(self):
        """
        Returns the element at the top of the callstack without removing it.

        :return: A CallFrame.
        """
        try:
            return self._callstack[-1]
        except IndexError:
            raise ValueError("Empty CallStack")

    @property
    def all_function_addresses(self):
        """
        Get all function addresses called in the path, from the earliest one to the most recent one

        :return: a list of function addresses
        :rtype: list
        """
        return [frame.caller_func_addr for frame in self._callstack]

    #
    # Public methods
    #

    def push(self, cf):
        """
        Push the :class:`CallFrame` `cf` on the callstack.
        """
        self._callstack.append(cf)

    def pop(self):
        """
        Pops one :class:`CallFrame` from the callstack.

        :return: A CallFrame.
        """
        try:
            return self._callstack.pop(-1)
        except IndexError:
            raise ValueError("Empty CallStack")

    def call(self, callsite_addr, addr, retn_target=None, stack_pointer=None):
        """
        Push a stack frame into the call stack. This method is called when calling a function in CFG recovery.

        :param int callsite_addr: Address of the call site
        :param int addr: Address of the call target
        :param int retn_target: Address of the return target
        :param int stack_pointer: Value of the stack pointer
        :return: None
        """

        frame = CallFrame(call_site_addr=callsite_addr, func_addr=addr, ret_addr=retn_target,
                          stack_ptr=stack_pointer)
        self._callstack.append(frame)

    def ret(self, retn_target):
        """
        Pop one or many call frames from the stack. This method is called when returning from a function in CFG
        recovery.

        :param int retn_target: The target to return to.
        :return: None
        """

        return_target_index = self._rfind_return_target(retn_target)

        if return_target_index is not None:
            # We may want to return to several levels up there, not only a
            # single stack frame
            levels = return_target_index

            # Remove all frames higher than the level
            self._callstack = self._callstack[: levels]

        else:
            l.warning("Returning to an unexpected address %#x", retn_target)

            # For Debugging
            # raise Exception()
            # There are cases especially in ARM where return is used as a jump
            # So we don't pop anything out

    def dbg_repr(self):
        """
        Debugging representation of this CallStack object.

        :return: Details of this CalLStack
        :rtype: str
        """

        stack = [ ]
        for i, frame in enumerate(reversed(self._callstack)):
            s = "%d | %s -> %s, returning to %s" % (
                i,
                "None" if frame.call_site_addr is None else "%#x" % frame.call_site_addr,
                "None" if frame.func_addr is None else "%#x" % frame.func_addr,
                "None" if frame.return_target is None else "%#x" % frame.return_target,
            )
            stack.append(s)

        return "\n".join(stack)

    def clear(self):
        """
        Clear the call stack.

        :return: None
        """
        self._callstack = [ ]

    def stack_suffix(self, context_sensitivity_level):
        """
        Generate the stack suffix. A stack suffix can be used as the key to a SimRun in CFG recovery.

        :param int context_sensitivity_level: Level of context sensitivity.
        :return: A tuple of stack suffix.
        :rtype: tuple
        """

        length = len(self._callstack)

        ret = ()
        for i in xrange(context_sensitivity_level):
            index = length - i - 1
            if index < 0:
                ret = (None, None) + ret
            else:
                frame = self._callstack[index]
                ret = (frame.call_site_addr, frame.call_target) + ret
        return ret

    def copy(self):
        c = CallStack()
        c._callstack = [cf.copy() for cf in self._callstack]
        return c

    #
    # Private methods
    #

    def _rfind_return_target(self, target):
        """
        Check if the return target exists in the stack, and return the index if exists. We always search from the most
        recent call stack frame since the most recent frame has a higher chance to be hit in normal CFG recovery.

        :param int target: Target of the return.
        :return: The index of the object
        :rtype: int
        """

        for i in xrange(len(self._callstack) - 1, -1, -1):
            frame = self._callstack[i]
            if frame.return_target == target:
                return i
        return None




import logging
import struct
import cffi
from collections import defaultdict

import networkx

from cle import ELF, PE, Blob, TLSObj
import pyvex
import simuvex
from claripy.utils.orderedset import OrderedSet

from ...knowledge import HookNode, BlockNode, FunctionManager
from ...analysis import Analysis
from ...errors import AngrCFGError
from ...extern_obj import AngrExternObject

from .cfg_node import CFGNode

l = logging.getLogger(name="angr.cfg_base")


class IndirectJump(object):
    def __init__(self, addr, ins_addr, func_addr, jumpkind, stmt_idx, resolved_targets=None, jumptable=False,
                 jumptable_addr=None, jumptable_entries=None):
        self.addr = addr
        self.ins_addr = ins_addr
        self.func_addr = func_addr
        self.jumpkind = jumpkind
        self.stmt_idx = stmt_idx
        self.resolved_targets = set() if resolved_targets is None else set(resolved_targets)
        self.jumptable = jumptable
        self.jumptable_addr = jumptable_addr
        self.jumptable_entries = jumptable_entries

    def __repr__(self):

        status = ""
        if self.jumptable:
            status = "jumptable"
            if self.jumptable_addr is not None:
                status += "@%#08x" % self.jumptable_addr
            if self.jumptable_entries is not None:
                status += " with %d entries" % self.jumptable_entries

        return "<IndirectJump %#08x - ins %#08x%s>" % (self.addr, self.ins_addr, " " + status if status else "")


class CFGBase(Analysis):
    """
    The base class for control flow graphs.
    """
    def __init__(self, sort, context_sensitivity_level, normalize=False, binary=None, force_segment=False, iropt_level=None):
        self.sort = sort
        self._context_sensitivity_level=context_sensitivity_level

        # Sanity checks
        if context_sensitivity_level < 0:
            raise ValueError("Unsupported context sensitivity level %d" % context_sensitivity_level)

        self._binary = binary if binary is not None else self.project.loader.main_bin
        self._force_segment = force_segment
        self._iropt_level = iropt_level

        # Initialization
        self._graph = None
        self._edge_map = None
        self._loop_back_edges = None
        self._overlapped_loop_headers = None
        self._thumb_addrs = set()

        # Traverse all the IRSBs, and put the corresponding CFGNode objects to a dict
        # CFGNodes dict indexed by block ID
        self._nodes = None
        # Lists of CFGNodes indexed by addresses of each block
        self._nodes_by_addr = None

        # Store all the functions analyzed before the set is cleared
        # Used for performance optimization
        self._changed_functions = None

        self._normalize = normalize
        # Flag, whether the CFG has been normalized or not
        self._normalized = False

        # IndirectJump object that describe all indirect exits found in the binary
        # stores as a map between addresses and IndirectJump objects
        self.indirect_jumps = {}

        # Get all executable memory regions
        self._exec_mem_regions = self._executable_memory_regions(None, self._force_segment)
        self._exec_mem_region_size = sum([(end - start) for start, end in self._exec_mem_regions])

        # initialize an UnresolvableTarget SimProcedure
        # but we do not want to hook the same symbol multiple times
        if not self.project.is_symbol_hooked('UnresolvableTarget'):
            ut_addr = self.project.hook_symbol('UnresolvableTarget',
                                               simuvex.SimProcedures['stubs']['UnresolvableTarget']
                                               )
        else:
            ut_addr = self.project.hooked_symbol_addr('UnresolvableTarget')
        self._unresolvable_target_addr = ut_addr

        # partially and fully analyzed functions
        # this is implemented as a state machine: jobs (CFGJob instances) of each function are put into
        # _jobs_to_analyze_per_function, which essentially makes the keys of this dict being all partially analyzed
        # functions so far. And then when a function does not have any more job to analyze in the future, it will be
        # put in to _completed_functions.

        # a dict of mapping between function addresses and sets of jobs (include both future jobs and pending jobs)
        # a set is used to speed up the job removal procedure
        self._jobs_to_analyze_per_function = defaultdict(set)
        # addresses of functions that have been completely recovered (i.e. all of its blocks are identified) so far
        self._completed_functions = set()

        # TODO: A segment tree to speed up CFG node lookups
        self._node_lookup_index = None
        self._node_lookup_index_warned = False

        self._ffi = cffi.FFI()

    def __contains__(self, cfg_node):
        return cfg_node in self._graph

    @property
    def normalized(self):
        return self._normalized

    @property
    def context_sensitivity_level(self):
        return self._context_sensitivity_level

    @property
    def functions(self):
        """
        A reference to the FunctionManager in the current knowledge base.

        :return: FunctionManager with all functions
        :rtype: angr.knowledge.FunctionManager
        """
        return self.kb.functions

    def _initialize_cfg(self):
        """
        Re-create the DiGraph
        """
        self._graph = networkx.DiGraph()

        self.kb.functions = FunctionManager(self.kb)

        self._jobs_to_analyze_per_function = defaultdict(set)
        self._completed_functions = set()

    def _post_analysis(self):

        if self._normalize:

            if not self._normalized:
                self.normalize()

            # Call normalize() on each function
            for f in self.kb.functions.values():
                if not self.project.is_hooked(f.addr):
                    f.normalize()

    def make_copy(self, copy_to):
        """
        Copy self attributes to the new object.

        :param CFGBase copy_to: The target to copy to.
        :return: None
        """

        copy_to._normalized = self._normalized

    # pylint: disable=no-self-use
    def copy(self):
        raise NotImplementedError()

    def output(self):
        raise NotImplementedError()

    def generate_index(self):
        """
        Generate an index of all nodes in the graph in order to speed up get_any_node() with anyaddr=True.

        :return: None
        """

        raise NotImplementedError("I'm too lazy to implement it right now")

    # TODO: Mark as deprecated
    def get_bbl_dict(self):
        return self._nodes

    def get_predecessors(self, cfgnode, excluding_fakeret=True, jumpkind=None):
        """
        Get predecessors of a node in the control flow graph.

        :param CFGNode cfgnode:             The node.
        :param bool excluding_fakeret:      True if you want to exclude all predecessors that is connected to the node
                                            with a fakeret edge.
        :param str or None jumpkind:        Only return predecessors with the specified jumpkind. This argument will be
                                            ignored if set to None.
        :return:                            A list of predecessors
        :rtype:                             list
        """

        if excluding_fakeret and jumpkind == 'Ijk_FakeRet':
            return [ ]

        if not excluding_fakeret and jumpkind is None:
            # fast path
            if cfgnode in self._graph:
                return self._graph.predecessors(cfgnode)
            return [ ]

        predecessors = []
        for pred, _, data in self._graph.in_edges_iter([cfgnode], data=True):
            jk = data['jumpkind']
            if jumpkind is not None:
                if jk == jumpkind:
                    predecessors.append(pred)
            elif excluding_fakeret:
                if jk != 'Ijk_FakeRet':
                    predecessors.append(pred)
            else:
                predecessors.append(pred)
        return predecessors

    def get_successors(self, basic_block, excluding_fakeret=True, jumpkind=None):
        """
        Get successors of a node in the control flow graph.

        :param CFGNode cfgnode:             The node.
        :param bool excluding_fakeret:      True if you want to exclude all successors that is connected to the node
                                            with a fakeret edge.
        :param str or None jumpkind:        Only return successors with the specified jumpkind. This argument will be
                                            ignored if set to None.
        :return:                            A list of successors
        :rtype:                             list
        """

        if jumpkind is not None:
            if excluding_fakeret and jumpkind == 'Ijk_FakeRet':
                return [ ]

        if not excluding_fakeret and jumpkind is None:
            # fast path
            if basic_block in self._graph:
                return self._graph.successors(basic_block)
            return [ ]

        successors = []
        for _, suc, data in self._graph.out_edges_iter([basic_block], data=True):
            jk = data['jumpkind']
            if jumpkind is not None:
                if jumpkind == jk:
                    successors.append(suc)
            elif excluding_fakeret:
                if jk != 'Ijk_FakeRet':
                    successors.append(suc)
            else:
                successors.append(suc)
        return successors

    def get_successors_and_jumpkind(self, basic_block, excluding_fakeret=True):
        successors = []
        for _, suc, data in self._graph.out_edges_iter([basic_block], data=True):
            if not excluding_fakeret or data['jumpkind'] != 'Ijk_FakeRet':
                successors.append((suc, data['jumpkind']))
        return successors

    def get_all_predecessors(self, cfgnode):
        """
        Get all predecessors of a specific node on the control flow graph.

        :param CFGNode cfgnode: The CFGNode object
        :return: A list of predecessors in the CFG
        :rtype: list
        """

        return networkx.dfs_predecessors(self._graph, cfgnode)

    def get_all_successors(self, basic_block):
        return networkx.dfs_successors(self._graph, basic_block)

    def get_node(self, block_id):
        """
        Get a single node from node key.

        :param BlockID block_id: Block ID of the node.
        :return:                 The CFGNode
        :rtype:                  CFGNode
        """
        if block_id in self._nodes:
            return self._nodes[block_id]
        return None

    def nodes(self):
        return self._graph.nodes()

    def get_any_node(self, addr, is_syscall=None, anyaddr=False):
        """
        Get an arbitrary CFGNode (without considering their contexts) from our graph.

        :param int addr:        Address of the beginning of the basic block. Set anyaddr to True to support arbitrary
                                address.
        :param bool is_syscall: Whether you want to get the syscall node or any other node. This is due to the fact that
                                syscall SimProcedures have the same address as the targer it returns to.
                                None means get either, True means get a syscall node, False means get something that isn't
                                a syscall node.
        :param bool anyaddr:    If anyaddr is True, then addr doesn't have to be the beginning address of a basic
                                block. By default the entire graph.nodes() will be iterated, and the first node
                                containing the specific address is returned, which is slow. If you need to do many such
                                queries, you may first call `generate_index()` to create some indices that may speed up the
                                query.
        :return: A CFGNode if there is any that satisfies given conditions, or None otherwise
        """

        # fastpath: directly look in the nodes list
        if not anyaddr and self._nodes_by_addr and \
                addr in self._nodes_by_addr and self._nodes_by_addr[addr]:
            return self._nodes_by_addr[addr][0]

        # slower path
        #if self._node_lookup_index is not None:
        #    pass

        # the slowest path
        # try to show a warning first
        # TODO: re-enable it once the segment tree is implemented
        #if self._node_lookup_index_warned == False:
        #    l.warning('Calling get_any_node() with anyaddr=True is slow on large programs. '
        #              'For better performance, you may first call generate_index() to generate some indices that may '
        #              'speed the node lookup.')
        #    self._node_lookup_index_warned = True

        for n in self.graph.nodes_iter():
            cond = n.looping_times == 0
            if anyaddr and n.size is not None:
                cond = cond and (addr >= n.addr and addr < n.addr + n.size)
            else:
                cond = cond and (addr == n.addr)
            if cond:
                if is_syscall is None:
                    return n
                if n.is_syscall == is_syscall:
                    return n

        return None

    def irsb_from_node(self, cfg_node):  # pylint:disable=unused-argument
        """
        Create an IRSB from a CFGNode object.
        """
        raise DeprecationWarning('"irsb_from_node()" is deprecated since SimIRSB does not exist anymore.')

    def get_any_irsb(self, addr):  # pylint:disable=unused-argument
        """
        Returns an IRSB of a certain address. If there are many IRSBs with the same address in CFG, return an arbitrary
        one.
        You should never assume this method returns a specific IRSB.

        :param int addr: Address of the IRSB to get.
        :return:         An arbitrary IRSB located at `addr`.
        :rtype:          simuvex.IRSB
        """
        raise DeprecationWarning('"get_any_irsb()" is deprecated since SimIRSB does not exist anymore.')

    def get_all_nodes(self, addr, is_syscall=None, anyaddr=False):
        """
        Get all CFGNodes whose address is the specified one.

        :param addr:       Address of the node
        :param is_syscall: True returns the syscall node, False returns the normal CFGNode, None returns both
        :return:           all CFGNodes
        """
        results = [ ]

        for cfg_node in self._graph.nodes_iter():
            if cfg_node.addr == addr or (anyaddr and cfg_node.addr <= addr < (cfg_node.addr + cfg_node.size)):
                if is_syscall and cfg_node.is_syscall:
                    results.append(cfg_node)
                elif is_syscall is False and not cfg_node.is_syscall:
                    results.append(cfg_node)
                else:
                    results.append(cfg_node)

        return results

    def nodes_iter(self):
        """
        An iterator of all nodes in the graph.

        :return: The iterator.
        :rtype: iterator
        """

        return self._graph.nodes_iter()

    def get_all_irsbs(self, addr):  # pylint:disable=unused-argument
        """
        Returns all IRSBs of a certain address, without considering contexts.
        """
        raise DeprecationWarning('"get_all_irsbs()" is deprecated since SimIRSB does not exist anymore.')

    def get_loop_back_edges(self):
        return self._loop_back_edges

    def get_branching_nodes(self):
        """
        Returns all nodes that has an out degree >= 2
        """
        nodes = set()
        for n in self._graph.nodes():
            if self._graph.out_degree(n) >= 2:
                nodes.add(n)
        return nodes

    def get_exit_stmt_idx(self, src_block, dst_block):
        """
        Get the corresponding exit statement ID for control flow to reach destination block from source block. The exit
        statement ID was put on the edge when creating the CFG.
        Note that there must be a direct edge between the two blocks, otherwise an exception will be raised.

        :return: The exit statement ID
        """

        if not self.graph.has_edge(src_block, dst_block):
            raise AngrCFGError('Edge (%s, %s) does not exist in CFG' % (src_block, dst_block))

        return self.graph[src_block][dst_block]['stmt_idx']

    @property
    def graph(self):
        return self._graph

    def remove_edge(self, block_from, block_to):
        edge = (block_from, block_to)

        if edge in self._graph:
            self._graph.remove_edge(*edge)

    def _to_snippet(self, cfg_node, jumpkind=None):
        """
        Convert a CFGNode instance to a CodeNode object.

        :param angr.analyses.CFGNode cfg_node: The CFGNode instance.
        :return: A converted CodeNode instance.
        :rtype: CodeNode
        """

        addr = cfg_node.addr

        if self.project.is_hooked(addr) and jumpkind != 'Ijk_NoHook':
            hooker = self.project._sim_procedures[addr]
            size = hooker.kwargs.get('length', 0)
            return HookNode(addr, size, hooker.procedure)

        return BlockNode(cfg_node.addr, cfg_node.size)  # pylint: disable=no-member

    def is_thumb_addr(self, addr):
        return addr in self._thumb_addrs

    def _arm_thumb_filter_jump_successors(self, addr, successors, get_ins_addr, get_exit_stmt_idx):
        """
        Filter successors for THUMB mode basic blocks, and remove those successors that won't be taken normally.

        :param int addr: Address of the basic block / SimIRSB.
        :param list successors: A list of successors.
        :param func get_ins_addr: A callable that returns the source instruction address for a successor.
        :param func get_stmt_idx: A callable that returns the source statement ID for a successor.
        :return: A new list of successors after filtering.
        :rtype: list
        """

        if not successors:
            return [ ]

        it_counter = 0
        conc_temps = {}
        can_produce_exits = set()
        bb = self.project.factory.block(addr, thumb=True, opt_level=0)

        for stmt in bb.vex.statements:
            if stmt.tag == 'Ist_IMark':
                if it_counter > 0:
                    it_counter -= 1
                    can_produce_exits.add(stmt.addr + stmt.delta)
            elif stmt.tag == 'Ist_WrTmp':
                val = stmt.data
                if val.tag == 'Iex_Const':
                    conc_temps[stmt.tmp] = val.con.value
            elif stmt.tag == 'Ist_Put':
                if stmt.offset == self.project.arch.registers['itstate'][0]:
                    val = stmt.data
                    if val.tag == 'Iex_RdTmp':
                        if val.tmp in conc_temps:
                            # We found an IT instruction!!
                            # Determine how many instructions are conditional
                            it_counter = 0
                            itstate = conc_temps[val.tmp]
                            while itstate != 0:
                                it_counter += 1
                                itstate >>= 8

        if it_counter != 0:
            l.debug('Basic block ends before calculated IT block (%#x)', addr)

        THUMB_BRANCH_INSTRUCTIONS = ('beq', 'bne', 'bcs', 'bhs', 'bcc', 'blo', 'bmi', 'bpl', 'bvs',
                                     'bvc', 'bhi', 'bls', 'bge', 'blt', 'bgt', 'ble', 'cbz', 'cbnz')
        for cs_insn in bb.capstone.insns:
            if cs_insn.mnemonic in THUMB_BRANCH_INSTRUCTIONS:
                can_produce_exits.add(cs_insn.address)

        successors_filtered = filter(lambda suc: get_ins_addr(suc) in can_produce_exits or
                                        get_exit_stmt_idx(suc) == 'default',
                            successors
                            )

        return successors_filtered

    def _executable_memory_regions(self, binary=None, force_segment=False):
        """
        Get all executable memory regions from the binaries

        :param binary: Binary object to collect regions from. If None, regions from all project binary objects are used.
        :param bool force_segment: Rely on binary segments instead of sections.
        :return: A sorted list of tuples (beginning_address, end_address)
        """

        if binary is None:
            binaries = self.project.loader.all_objects
        else:
            binaries = [ binary ]

        memory_regions = [ ]

        for b in binaries:
            rebase_addr = b.rebase_addr

            if isinstance(b, ELF):
                # If we have sections, we get result from sections
                if not force_segment and b.sections:
                    # Get all executable sections
                    for section in b.sections:
                        if section.is_executable:
                            tpl = (rebase_addr + section.min_addr, rebase_addr + section.max_addr)
                            memory_regions.append(tpl)

                else:
                    # Get all executable segments
                    for segment in b.segments:
                        if segment.is_executable:
                            tpl = (rebase_addr + segment.min_addr, rebase_addr + segment.max_addr)
                            memory_regions.append(tpl)

            elif isinstance(b, PE):
                for section in b.sections:
                    if section.is_executable:
                        tpl = (rebase_addr + section.min_addr, rebase_addr + section.max_addr)
                        memory_regions.append(tpl)

            elif isinstance(b, Blob):
                # a blob is entirely executable
                tpl = (rebase_addr + b.get_min_addr(), rebase_addr + b.get_max_addr())
                memory_regions.append(tpl)

            elif isinstance(b, (AngrExternObject, TLSObj)):
                pass

            else:
                l.warning('Unsupported object format "%s". Treat it as an executable.', b.__class__.__name__)

                tpl = (rebase_addr + b.get_min_addr(), rebase_addr + b.get_max_addr())
                memory_regions.append(tpl)

        if not memory_regions:
            memory_regions = [
                (self.project.loader.main_bin.rebase_addr + start,
                 self.project.loader.main_bin.rebase_addr + start + len(cbacker)
                 )
                for start, cbacker in self.project.loader.memory.cbackers
                ]

        memory_regions = sorted(memory_regions, key=lambda x: x[0])

        return memory_regions

    def _addr_in_exec_memory_regions(self, addr):
        """
        Test if the address belongs to an executable memory region.

        :param int addr: The address to test
        :return: True if the address belongs to an exectubale memory region, False otherwise
        :rtype: bool
        """

        for start, end in self._exec_mem_regions:
            if start <= addr < end:
                return True
        return False

    def _addr_belongs_to_section(self, addr):
        """
        Return the section object that the address belongs to.

        :param int addr: The address to test
        :return: The section that the address belongs to, or None if the address does not belong to any section, or if
                section information is not available.
        :rtype: cle.Section
        """

        obj = self.project.loader.addr_belongs_to_object(addr)

        if obj is None:
            return None

        if isinstance(obj, AngrExternObject):
            # the address is from a section allocated by angr.
            return None

        return obj.find_section_containing(addr)

    def _addrs_belong_to_same_section(self, addr_a, addr_b):
        """
        Test if two addresses belong to the same section.

        :param int addr_a:  The first address to test.
        :param int addr_b:  The second address to test.
        :return:            True if the two addresses belong to the same section or both of them do not belong to any
                            section, False otherwise.
        :rtype:             bool
        """

        obj = self.project.loader.addr_belongs_to_object(addr_a)

        if obj is None:
            # test if addr_b also does not belong to any object
            obj_b = self.project.loader.addr_belongs_to_object(addr_b)
            if obj_b is None:
                return True
            return False

        src_section = obj.find_section_containing(addr_a)
        if src_section is None:
            # test if addr_b also does not belong to any section
            dst_section = obj.find_section_containing(addr_b)
            if dst_section is None:
                return True
            return False

        return src_section.contains_addr(addr_b - obj.rebase_addr)

    def _addr_next_section(self, addr):
        """
        Return the next section object after the given address.

        :param int addr: The address to test
        :return: The next section that goes after the given address, or None if there is no section after the address,
                 or if section information is not available.
        :rtype: cle.Section
        """

        obj = self.project.loader.addr_belongs_to_object(addr)

        if obj is None:
            return None

        if isinstance(obj, AngrExternObject):
            # the address is from a section allocated by angr.
            return None

        for section in obj.sections:
            start = section.vaddr + obj.rebase_addr

            if addr < start:
                return section

        return None

    def _addr_belongs_to_segment(self, addr):
        """
        Return the section object that the address belongs to.

        :param int addr: The address to test
        :return: The section that the address belongs to, or None if the address does not belong to any section, or if
                section information is not available.
        :rtype: cle.Segment
        """

        obj = self.project.loader.addr_belongs_to_object(addr)

        if obj is None:
            return None

        if isinstance(obj, AngrExternObject):
            # the address is from a section allocated by angr.
            return None

        return obj.find_segment_containing(addr)

    def _addr_hooked_or_syscall(self, addr):
        """
        Check whether the address belongs to a hook or a syscall.

        :param int addr:    The address to check.
        :return:            True if the address is hooked or belongs to a syscall. False otherwise.
        :rtype:             bool
        """

        return self.project.is_hooked(addr) or self.project._simos.syscall_table.get_by_addr(addr) is not None

    def _fast_memory_load(self, addr):
        """
        Perform a fast memory loading of static content from static regions, a.k.a regions that are mapped to the
        memory by the loader.

        :param int addr: Address to read from.
        :return: The data, or None if the address does not exist.
        :rtype: cffi.CData
        """

        try:
            buff, _ = self.project.loader.memory.read_bytes_c(addr)
            return buff

        except KeyError:
            return None

    def _fast_memory_load_pointer(self, addr):
        """
        Perform a fast memory loading of a pointer.

        :param int addr: Address to read from.
        :return:         A pointer.
        :rtype:          int
        """

        pointer_size = self.project.arch.bits / 8
        buf = self._fast_memory_load(addr)

        if self.project.arch.memory_endness == 'Iend_LE':
            fmt = "<"
        else:
            fmt = ">"
        if pointer_size == 8:
            fmt += "Q"
        elif pointer_size == 4:
            fmt += "I"
        else:
            raise AngrCFGError("Pointer size of %d is not supported", pointer_size)

        ptr_str = self._ffi.unpack(self._ffi.cast('char*', buf), pointer_size)
        ptr = struct.unpack(fmt, ptr_str)[0]  # type:int

        return ptr

    #
    # Analyze function features
    #

    def _analyze_function_features(self):
        """
        For each function in the function_manager, try to determine if it returns or not. A function does not return if
        it calls another function that is known to be not returning, and this function does not have other exits.

        We might as well analyze other features of functions in the future.

        A function does not return if
        a) it is a SimProcedure that has NO_RET being True,
        or
        b) it is completely recovered (i.e. every block of this function has been recovered, and no future block will
           be added to it), and it does not have a ret or any equivalent instruction.

        A function returns if any of its block contains a ret instruction or any equivalence.
        """

        changes = {
            'functions_return': [],
            'functions_do_not_return': []
        }

        if self._changed_functions is not None:
            all_functions = self._changed_functions
            caller_functions = set()

            for func_addr in self._changed_functions:
                if func_addr not in self.kb.functions.callgraph:
                    continue
                callers = self.kb.functions.callgraph.predecessors(func_addr)
                for f in callers:
                    caller_functions.add(f)

            all_functions |= caller_functions

            all_functions = [ self.kb.functions.function(addr=f) for f in all_functions if f in self.kb.functions ]

        else:
            all_functions = self.kb.functions.values()

        # pylint: disable=too-many-nested-blocks
        for func in all_functions:  # type: angr.knowledge.Function

            if func.returning is not None:
                # It has been determined before. Skip it
                continue

            # If there is at least one return site, then this function is definitely returning
            if func.has_return:
                changes['functions_return'].append(func)
                func.returning = True
                continue

            # This function does not have endpoints. It's either because it does not return, or we haven't analyzed all
            # blocks of it.

            # Let's first see if it's a known SimProcedure that does not return
            if self.project.is_hooked(func.addr):
                hooker = self.project.hooked_by(func.addr)
                procedure = hooker.procedure
            else:
                syscall = self.project._simos.syscall_table.get_by_addr(func.addr)
                if syscall is not None:
                    procedure = syscall.simproc
                else:
                    procedure = None

            if procedure is not None and hasattr(procedure, 'NO_RET'):
                if procedure.NO_RET:
                    func.returning = False
                    changes['functions_do_not_return'].append(func)
                else:
                    func.returning = True
                    changes['functions_return'].append(func)
                continue

            # did we finish analyzing this function?
            if func.addr not in self._completed_functions:
                continue

            if not func.block_addrs_set:
                # there is no block inside this function
                # it might happen if the function has been incorrectly identified as part of another function
                # the error will be corrected during post-processing. In fact at this moment we cannot say anything
                # about whether this function returns or not. We always assume it returns.
                func.returning = True
                changes['functions_return'].append(func)
                continue

            bail_out = False

            # if this function has jump out sites, it returns as long as any of the target function returns
            for jump_out_site in func.jumpout_sites:

                if func.returning:
                    # if there are multiple jump out sites and we have determined the "returning status" from one of
                    # the jump out sites, we can exit the loop early
                    continue
                jump_out_site_successors = jump_out_site.successors()
                if not jump_out_site_successors:
                    # not sure where it jumps to. bail out
                    bail_out = True
                    continue
                jump_out_target = jump_out_site_successors[0]
                if not self.kb.functions.contains_addr(jump_out_target.addr):
                    # wait it does not jump to a function?
                    bail_out = True
                    continue
                target_func = self.kb.functions[jump_out_target.addr]
                if target_func.returning is True:
                    func.returning = True
                    changes['functions_return'].append(func)
                    bail_out = True
                elif target_func.returning is None:
                    # the returning status of at least one of the target functions is not decided yet.
                    bail_out = True

            if bail_out:
                # bail out
                continue

            # well this function does not return then
            func.returning = False
            changes['functions_do_not_return'].append(func)

        return changes

    def _iteratively_analyze_function_features(self):
        """
        Iteratively analyze function features until a fixed point is reached.

        :return: the "changes" dict
        :rtype:  dict
        """

        changes = {
            'functions_do_not_return': set(),
            'functions_return': set()
        }

        while True:
            new_changes = self._analyze_function_features()

            changes['functions_do_not_return'] |= set(new_changes['functions_do_not_return'])
            changes['functions_return'] |= set(new_changes['functions_return'])

            if not new_changes['functions_do_not_return'] and not new_changes['functions_return']:
                # a fixed point is reached
                break

        return changes

    def normalize(self):
        """
        Normalize the CFG, making sure that there are no overlapping basic blocks.

        Note that this method will not alter transition graphs of each function in self.kb.functions. You may call
        normalize() on each Function object to normalize their transition graphs.

        :return: None
        """

        graph = self.graph

        smallest_nodes = { }  # indexed by end address of the node
        end_addresses_to_nodes = defaultdict(set)

        for n in graph.nodes():
            if n.is_simprocedure:
                continue
            end_addr = n.addr + n.size
            key = (end_addr, n.callstack_key)
            # add the new item
            end_addresses_to_nodes[key].add(n)

        for key in end_addresses_to_nodes.keys():
            if len(end_addresses_to_nodes[key]) == 1:
                smallest_nodes[key] = next(iter(end_addresses_to_nodes[key]))
                del end_addresses_to_nodes[key]

        while end_addresses_to_nodes:
            key_to_find = (None, None)
            for tpl, x in end_addresses_to_nodes.iteritems():
                if len(x) > 1:
                    key_to_find = tpl
                    break

            end_addr, callstack_key = key_to_find
            all_nodes = end_addresses_to_nodes[key_to_find]

            all_nodes = sorted(all_nodes, key=lambda node: node.addr, reverse=True)
            smallest_node = all_nodes[0] # take the one that has the highest address
            other_nodes = all_nodes[1:]

            self._normalize_core(graph, callstack_key, smallest_node, other_nodes, smallest_nodes,
                                 end_addresses_to_nodes
                                 )

            del end_addresses_to_nodes[key_to_find]
            # make sure the smallest node is stored in end_addresses
            smallest_nodes[key_to_find] = smallest_node

            # corner case
            # sometimes two overlapping blocks may not be ending at the instruction. this might happen when one of the
            # blocks (the bigger one) hits the instruction count limit or bytes limit before reaching the end address
            # of the smaller block. in this case we manually pick up those blocks.
            if not end_addresses_to_nodes:
                # find if there are still overlapping blocks
                sorted_smallest_nodes = defaultdict(list)  # callstack_key is the key of this dict
                for k, node in smallest_nodes.iteritems():
                    _, callstack_key = k
                    sorted_smallest_nodes[callstack_key].append(node)
                for k in sorted_smallest_nodes.iterkeys():
                    sorted_smallest_nodes[k] = sorted(sorted_smallest_nodes[k], key=lambda node: node.addr)

                for callstack_key, lst in sorted_smallest_nodes.iteritems():
                    lst_len = len(lst)
                    for i, node in enumerate(lst):
                        if i == lst_len - 1:
                            break
                        next_node = lst[i + 1]
                        if node.addr <= next_node.addr < node.addr + node.size:
                            # umm, those nodes are overlapping, but they must have different end addresses
                            nodekey_a = node.addr + node.size, callstack_key
                            nodekey_b = next_node.addr + next_node.size, callstack_key

                            if nodekey_a in smallest_nodes and nodekey_b in smallest_nodes:
                                # misuse end_addresses_to_nodes
                                end_addresses_to_nodes[(node.addr + node.size, callstack_key)].add(node)
                                end_addresses_to_nodes[(node.addr + node.size, callstack_key)].add(next_node)

                            smallest_nodes.pop(nodekey_a, None)
                            smallest_nodes.pop(nodekey_b, None)

        self._normalized = True

    def _normalize_core(self, graph, callstack_key, smallest_node, other_nodes, smallest_nodes, end_addresses_to_nodes):

        # Break other nodes
        for n in other_nodes:
            new_size = smallest_node.addr - n.addr
            if new_size == 0:
                # This node has the same size as the smallest one. Don't touch it.
                continue

            new_end_addr = n.addr + new_size

            # Does it already exist?
            new_node = None
            key = (new_end_addr, n.callstack_key)
            # the logic below is a little convoluted. we check if key exists in either end_address_to_nodes or
            # smallest_nodes, since we don't always add the new node back to end_addresses_to_nodes dict - we only do so
            # when there are more than one node with that key.
            if key in end_addresses_to_nodes:
                new_node = next((i for i in end_addresses_to_nodes[key] if i.addr == n.addr), None)
            if new_node is None:
                if key in smallest_nodes and smallest_nodes[key].addr == n.addr:
                    new_node = smallest_nodes[key]

            if new_node is None:
                # Create a new one
                new_node = CFGNode(n.addr, new_size, self, callstack_key=callstack_key,
                                   function_address=n.function_address, block_id=n.block_id,
                                   instruction_addrs=[i for i in n.instruction_addrs
                                                      if n.addr <= i <= n.addr + new_size
                                                      ]
                                   )

                # Copy instruction addresses
                new_node.instruction_addrs = [ins_addr for ins_addr in n.instruction_addrs
                                              if ins_addr < n.addr + new_size]
                # Put the new node into end_addresses list
                if key in smallest_nodes:
                    end_addresses_to_nodes[key].add(smallest_nodes[key])
                    end_addresses_to_nodes[key].add(new_node)
                else:
                    smallest_nodes[key] = new_node

            # Modify the CFG
            original_predecessors = list(graph.in_edges_iter([n], data=True))
            original_successors = list(graph.out_edges_iter([n], data=True))

            if smallest_node not in graph:
                continue

            for _, d, data in original_successors:
                if d not in graph[smallest_node]:
                    if d is n:
                        graph.add_edge(smallest_node, new_node, **data)
                    else:
                        graph.add_edge(smallest_node, d, **data)

            for p, _, _ in original_predecessors:
                graph.remove_edge(p, n)
            if n in graph:
                graph.remove_node(n)

            # Update nodes dict
            self._nodes[n.block_id] = new_node
            if n in self._nodes_by_addr[n.addr]:
                self._nodes_by_addr[n.addr] = filter(lambda x,the_node=n: x is not the_node,
                                                     self._nodes_by_addr[n.addr]
                                                     )
                self._nodes_by_addr[n.addr].append(new_node)

            for p, _, data in original_predecessors:
                # Consider the following case: two basic blocks ending at the same position, where A is larger, and
                # B is smaller. Suppose there is an edge going from the end of A to A itself, and apparently there
                # is another edge from B to A as well. After splitting A into A' and B, we DO NOT want to add A back
                # in, otherwise there will be an edge from A to A`, while A should totally be got rid of in the new
                # graph.
                if p not in other_nodes:
                    graph.add_edge(p, new_node, data)

            # We should find the correct successor
            new_successors = [i for i in [smallest_node] + other_nodes
                              if i.addr == smallest_node.addr]
            if new_successors:
                new_successor = new_successors[0]
                graph.add_edge(new_node, new_successor, jumpkind='Ijk_Boring')
            else:
                # We gotta create a new one
                l.error('normalize(): Please report it to Fish.')

    #
    # Job management
    #

    def _register_analysis_job(self, func_addr, job):
        """
        Register an analysis job of a function to job manager. This allows us to track whether we have finished
        analyzing/recovering a function or not.

        :param int func_addr: Address of the function that this job belongs to.
        :param job:           The job to register. Note that it does not necessarily be the a CFGJob instance. There
                              can be PendingJob or PendingJob or other instances, too.
        :return:              None
        """

        self._jobs_to_analyze_per_function[func_addr].add(job)

    def _deregister_analysis_job(self, func_addr, job):
        """
        Deregister/Remove an analysis job of a function from job manager.

        :param int func_addr: Address of the function that this job belongs to.
        :param job:           The job to deregister.
        :return:              None
        """

        self._jobs_to_analyze_per_function[func_addr].remove(job)

    def _get_finished_functions(self):
        """
        Obtain all functions of which we have finished analyzing. As _jobs_to_analyze_per_function is a defaultdict(),
        if a function address shows up in it with an empty job list, we consider we have exhausted all jobs of this
        function (both current jobs and pending jobs), thus the analysis of this function is done.

        :return: a list of function addresses of that we have finished analysis.
        :rtype:  list
        """

        finished_func_addrs = [ ]
        for func_addr, all_jobs in self._jobs_to_analyze_per_function.iteritems():
            if not all_jobs:
                # great! we have finished analyzing this function!
                finished_func_addrs.append(func_addr)

        return finished_func_addrs

    def _cleanup_analysis_jobs(self, finished_func_addrs=None):
        """
        From job manager, remove all functions of which we have finished analysis.

        :param list or None finished_func_addrs: A list of addresses of functions of which we have finished analysis.
                                                 A new list of function addresses will be obtained by calling
                                                 _get_finished_functions() if this parameter is None.
        :return:                                 None
        """

        if finished_func_addrs is None:
            finished_func_addrs = self._get_finished_functions()

        for func_addr in finished_func_addrs:
            if func_addr in self._jobs_to_analyze_per_function:
                del self._jobs_to_analyze_per_function[func_addr]

    def _make_completed_functions(self):
        """
        Fill in self._completed_functions list and clean up job manager.

        :return: None
        """

        finished = self._get_finished_functions()
        for func_addr in finished:
            self._completed_functions.add(func_addr)
        self._cleanup_analysis_jobs(finished_func_addrs=finished)

    #
    # Function identification and such
    #

    def remove_function_alignments(self):
        """
        Remove all function alignments.

        :return: None
        """

        for func_addr in self.kb.functions.keys():
            function = self.kb.functions[func_addr]
            if function.is_simprocedure or function.is_syscall:
                continue
            if len(function.block_addrs_set) == 1:
                block = next((b for b in function.blocks), None)
                if block is None:
                    continue
                if all(self._is_noop_insn(insn) for insn in block.capstone.insns):
                    # remove this function
                    l.debug('Function chunk %#x is used as function alignments. Removing it.', func_addr)
                    del self.kb.functions[func_addr]

    def make_functions(self):
        """
        Revisit the entire control flow graph, create Function instances accordingly, and correctly put blocks into
        each function.

        Although Function objects are crated during the CFG recovery, they are neither sound nor accurate. With a
        pre-constructed CFG, this method rebuilds all functions bearing the following rules:

            - A block may only belong to one function.
            - Small functions lying inside the startpoint and the endpoint of another function will be merged with the
              other function
            - Tail call optimizations are detected.
            - PLT stubs are aligned by 16.

        :return: None
        """

        # TODO: Is it required that PLT stubs are always aligned by 16? If so, on what architectures and platforms is it
        # TODO:  enforced?

        tmp_functions = self.kb.functions.copy()

        for function in tmp_functions.values():
            function.mark_nonreturning_calls_endpoints()

        # Clear old functions dict
        self.kb.functions.clear()

        blockaddr_to_function = { }
        traversed_cfg_nodes = set()

        function_nodes = set()

        # Find nodes for beginnings of all functions
        for _, dst, data in self.graph.edges_iter(data=True):
            jumpkind = data.get('jumpkind', "")
            if jumpkind == 'Ijk_Call' or jumpkind.startswith('Ijk_Sys'):
                function_nodes.add(dst)

        entry_node = self.get_any_node(self._binary.entry)
        if entry_node is not None:
            function_nodes.add(entry_node)

        # aggressively remove and merge functions
        # For any function, if there is a call to it, it won't be removed
        removed_functions = self._process_irrational_functions(tmp_functions,
                                                               set([n.addr for n in function_nodes]),
                                                               blockaddr_to_function
                                                               )

        for n in self.graph.nodes_iter():
            if n.addr in tmp_functions or n.addr in removed_functions:
                function_nodes.add(n)

        # traverse the graph starting from each node, not following call edges
        # it's important that we traverse all functions in order so that we have a greater chance to come across
        # rational functions before its irrational counterparts (e.g. due to failed jump table resolution)

        min_stage_2_progress = 50.0
        max_stage_2_progress = 90.0
        nodes_count = len(function_nodes)
        for i, fn in enumerate(sorted(function_nodes, key=lambda n: n.addr)):

            if self._show_progressbar or self._progress_callback:
                progress = min_stage_2_progress + (max_stage_2_progress - min_stage_2_progress) * (i * 1.0 / nodes_count)
                self._update_progress(progress)

            self._graph_bfs_custom(self.graph, [ fn ], self._graph_traversal_handler, blockaddr_to_function,
                                   tmp_functions, traversed_cfg_nodes
                                   )

        # Don't forget those small function chunks that are not called by anything.
        # There might be references to them from data, or simply references that we cannot find via static analysis

        secondary_function_nodes = set()
        # add all function chunks ("functions" that are not called from anywhere)
        for func_addr in tmp_functions:
            node = self.get_any_node(func_addr)
            if node is None:
                continue
            if node.addr not in blockaddr_to_function:
                secondary_function_nodes.add(node)

        missing_cfg_nodes = set(self.graph.nodes()) - traversed_cfg_nodes
        missing_cfg_nodes = { node for node in missing_cfg_nodes if node.function_address is not None }
        if missing_cfg_nodes:
            l.debug('%d CFGNodes are missing in the first traversal.', len(missing_cfg_nodes))
            secondary_function_nodes |=  missing_cfg_nodes

        min_stage_3_progress = 90.0
        max_stage_3_progress = 99.9

        nodes_count = len(secondary_function_nodes)
        for i, fn in enumerate(sorted(secondary_function_nodes, key=lambda n: n.addr)):

            if self._show_progressbar or self._progress_callback:
                progress = min_stage_3_progress + (max_stage_3_progress - min_stage_3_progress) * (i * 1.0 / nodes_count)
                self._update_progress(progress)

            self._graph_bfs_custom(self.graph, [fn], self._graph_traversal_handler, blockaddr_to_function,
                                   tmp_functions
                                   )

        # Remove all stubs after PLT entries
        to_remove = set()
        for fn in self.kb.functions.values():
            addr = fn.addr - (fn.addr % 16)
            if addr != fn.addr and addr in self.kb.functions and self.kb.functions[addr].is_plt:
                to_remove.add(fn.addr)

        # remove empty functions
        for function in self.kb.functions.values():
            if function.startpoint is None:
                to_remove.add(function.addr)

        for addr in to_remove:
            del self.kb.functions[addr]

        # Update CFGNode.function_address
        for node in self._nodes.itervalues():
            if node.addr in blockaddr_to_function:
                node.function_address = blockaddr_to_function[node.addr].addr

        # mark endpoints
        for function in self.kb.functions.values():
            function.mark_nonreturning_calls_endpoints()

    def _process_irrational_functions(self, functions, predetermined_function_addrs, blockaddr_to_function):
        """
        For unresolveable indirect jumps, angr marks those jump targets as individual functions. For example, usually
        the following pattern is seen:

        sub_0x400010:
            push ebp
            mov esp, ebp
            ...
            cmp eax, 10
            ja end
            mov eax, jumptable[eax]
            jmp eax

        sub_0x400080:
            # do something here
            jmp end

        end (0x400e00):
            pop ebp
            ret

        In the example above, `process_irrational_functions` will remove function 0x400080, and merge it with function
        0x400010.

        :param angr.knowledge.FunctionManager functions: all functions that angr recovers, including those ones that are
            misidentified as functions.
        :param dict blockaddr_to_function: A mapping between block addresses and Function instances.
        :return: a list of addresses of all removed functions
        :rtype: list
        """

        functions_to_remove = { }

        functions_can_be_removed = set(functions.keys()) - set(predetermined_function_addrs)

        for func_addr, function in functions.iteritems():

            if func_addr in functions_to_remove:
                continue

            # check all blocks and see if any block ends with an indirect jump and is not resolved
            has_unresolved_jumps = False
            # the functions to merge with must be locating between the unresolved basic block address and the endpoint
            # of the current function
            max_unresolved_jump_addr = 0
            for block_addr in function.block_addrs_set:
                if block_addr in self.indirect_jumps and \
                        self.indirect_jumps[block_addr].jumpkind == 'Ijk_Boring' and \
                        not self.indirect_jumps[block_addr].resolved_targets:
                    # it's not resolved
                    # we should also make sure it's a jump, not a call
                    has_unresolved_jumps = True
                    max_unresolved_jump_addr = max(max_unresolved_jump_addr, block_addr)

            if not has_unresolved_jumps:
                continue

            if function.startpoint is None:
                continue

            startpoint_addr = function.startpoint.addr
            if not function.endpoints:
                # Function should have at least one endpoint
                continue
            endpoint_addr = max([ a.addr for a in function.endpoints ])
            the_endpoint = next(a for a in function.endpoints if a.addr == endpoint_addr)
            endpoint_addr += the_endpoint.size

            # sanity check: startpoint of the function should be greater than its endpoint
            if startpoint_addr >= endpoint_addr:
                continue
            if max_unresolved_jump_addr >= endpoint_addr:
                continue

            # scan forward from the endpoint to include any function tail jumps
            # Here is an example:
            # loc_8049562:
            #       mov eax, ebp
            #       add esp, 3ch
            #       ...
            #       ret
            # loc_804956c:
            #       mov ebp, 3
            #       jmp loc_8049562
            # loc_8049573:
            #       mov ebp, 4
            #       jmp loc_8049562
            #
            last_addr = endpoint_addr
            tmp_state = self.project.factory.blank_state(mode='fastpath')
            while True:
                try:
                    # using successors is slow, but acceptable since we won't be creating millions of blocks here...
                    tmp_state.ip = last_addr
                    b = self.project.factory.successors(tmp_state, jumpkind='Ijk_Boring')
                    if len(b.successors) != 1:
                        break
                    if b.successors[0].scratch.jumpkind != 'Ijk_Boring':
                        break
                    if b.successors[0].ip.symbolic:
                        break
                    suc_addr = b.successors[0].ip._model_concrete
                    if max(startpoint_addr, the_endpoint.addr - 0x40) <= suc_addr < the_endpoint.addr + the_endpoint.size:
                        # increment the endpoint_addr
                        endpoint_addr = b.addr + b.artifacts['irsb_size']
                    else:
                        break

                    last_addr = b.addr + b.artifacts['irsb_size']

                except (simuvex.SimTranslationError, simuvex.SimMemoryError, simuvex.SimIRSBError, simuvex.SimEngineError):
                    break

            # find all functions that are between [ startpoint, endpoint ]

            should_merge = True
            functions_to_merge = set()
            for f_addr in functions_can_be_removed:
                f = functions[f_addr]
                if f_addr == func_addr:
                    continue
                if max_unresolved_jump_addr < f_addr < endpoint_addr and \
                        all([max_unresolved_jump_addr < b_addr < endpoint_addr for b_addr in f.block_addrs]):
                    if f_addr in functions_to_remove:
                        # this function has already been merged with other functions before... it cannot be merged with
                        # this function anymore
                        should_merge = False
                        break
                    if f_addr in predetermined_function_addrs:
                        # this function is a legit one. it shouldn't be removed/merged
                        should_merge = False
                        break
                    functions_to_merge.add(f_addr)

            if not should_merge:
                # we shouldn't merge...
                continue

            for f_addr in functions_to_merge:
                functions_to_remove[f_addr] = func_addr
                continue

        # merge all functions
        for to_remove, merge_with in functions_to_remove.iteritems():
            func_merge_with = self._addr_to_function(merge_with, blockaddr_to_function, functions)

            for block_addr in functions[to_remove].block_addrs:
                blockaddr_to_function[block_addr] = func_merge_with

            del functions[to_remove]

        return functions_to_remove.keys()

    def _addr_to_function(self, addr, blockaddr_to_function, known_functions):
        """
        Convert an address to a Function object, and store the mapping in a dict. If the block is known to be part of a
        function, just return that function.

        :param int addr: Address to convert
        :param dict blockaddr_to_function: A mapping between block addresses to Function instances.
        :param angr.knowledge.FunctionManager known_functions: Recovered functions.
        :return: a Function object
        :rtype: angr.knowledge.Function
        """

        if addr in blockaddr_to_function:
            f = blockaddr_to_function[addr]
        else:
            is_syscall = False
            syscall = self.project._simos.syscall_table.get_by_addr(addr)
            if syscall is not None:
                is_syscall = True

            n = self.get_any_node(addr, is_syscall=is_syscall)
            if n is None: node = addr
            else: node = self._to_snippet(n)

            self.kb.functions._add_node(addr, node, syscall=is_syscall)
            f = self.kb.functions.function(addr=addr)

            blockaddr_to_function[addr] = f

            if addr in known_functions:
                f.returning = known_functions.function(addr).returning
            else:
                # TODO:
                pass

        return f

    def _graph_bfs_custom(self, g, starts, callback, blockaddr_to_function, known_functions, traversed_cfg_nodes=None):
        """
        A customized control flow graph BFS implementation with the following rules:
        - Call edges are not followed.
        - Syscall edges are not followed.

        :param networkx.DiGraph g: The graph.
        :param list starts: A collection of beginning nodes to start graph traversal.
        :param func callback: Callback function for each edge and node.
        :param dict blockaddr_to_function: A mapping between block addresses to Function instances.
        :param angr.knowledge.FunctionManager known_functions: Already recovered functions.
        :param set traversed_cfg_nodes: A set of CFGNodes that are traversed before.
        :return: None
        """

        stack = OrderedSet(starts)
        traversed = set() if traversed_cfg_nodes is None else traversed_cfg_nodes

        while stack:
            n = stack.pop(last=False)  # type: CFGNode

            if n in traversed:
                continue

            traversed.add(n)

            if n.has_return:
                callback(n, None, {'jumpkind': 'Ijk_Ret'}, blockaddr_to_function, known_functions, None)

            elif g.out_degree(n) == 0:
                # it's a single node
                callback(n, None, None, blockaddr_to_function, known_functions, None)

            else:
                all_out_edges = g.out_edges(n, data=True)
                for src, dst, data in all_out_edges:
                    callback(src, dst, data, blockaddr_to_function, known_functions, all_out_edges)

                    jumpkind = data.get('jumpkind', "")
                    if not (jumpkind == 'Ijk_Call' or jumpkind.startswith('Ijk_Sys')):
                        # Only follow none call edges
                        if dst not in stack and dst not in traversed:
                            stack.add(dst)

    def _graph_traversal_handler(self, src, dst, data, blockaddr_to_function, known_functions, all_edges):
        """
        Graph traversal handler. It takes in a node or an edge, and create new functions or add nodes to existing
        functions accordingly. Oh, it also create edges on the transition map of functions.

        :param CFGNode src: Beginning of the edge, or a single node when dst is None.
        :param CFGNode dst: Destination of the edge. For processing a single node, `dst` is None.
        :param dict data: Edge data in the CFG. 'jumpkind' should be there if it's not None.
        :param dict blockaddr_to_function: A mapping between block addresses to Function instances.
        :param angr.knowledge.FunctionManager known_functions: Already recovered functions.
        :param list or None all_edges: All edges going out from src.
        :return: None
        """

        src_addr = src.addr
        src_function = self._addr_to_function(src_addr, blockaddr_to_function, known_functions)

        if src_addr not in src_function.block_addrs_set:
            n = self.get_any_node(src_addr)
            if n is None: node = src_addr
            else: node = self._to_snippet(n)
            self.kb.functions._add_node(src_function.addr, node)

        if data is None:
            # it's a single node only
            return

        jumpkind = data['jumpkind']

        if jumpkind == 'Ijk_Ret':
            n = self.get_any_node(src_addr)
            if n is None: from_node = src_addr
            else: from_node = self._to_snippet(n)
            self.kb.functions._add_return_from(src_function.addr, from_node, None)

        if dst is None:
            return

        dst_addr = dst.addr

        # get instruction address and statement index
        ins_addr = data.get('ins_addr', None)
        stmt_idx = data.get('stmt_idx', None)

        if jumpkind == 'Ijk_Call' or jumpkind.startswith('Ijk_Sys'):

            is_syscall = jumpkind.startswith('Ijk_Sys')

            # It must be calling a function
            dst_function = self._addr_to_function(dst_addr, blockaddr_to_function, known_functions)

            n = self.get_any_node(src_addr)
            if n is None: src_node = src_addr
            else: src_node = self._to_snippet(n)

            fakeret_node = None if not all_edges else self._one_fakeret_node(all_edges)
            fakeret_addr = fakeret_node.addr if fakeret_node is not None else None

            self.kb.functions._add_call_to(src_function.addr, src_node, dst_addr, fakeret_addr, syscall=is_syscall,
                                           ins_addr=ins_addr, stmt_idx=stmt_idx)

            if dst_function.returning:
                returning_target = src.addr + src.size
                if returning_target not in blockaddr_to_function:
                    if returning_target not in known_functions:
                        blockaddr_to_function[returning_target] = src_function
                    else:
                        self._addr_to_function(returning_target, blockaddr_to_function, known_functions)

                to_outside = not blockaddr_to_function[returning_target] is src_function

                n = self.get_any_node(returning_target)
                if n is None: returning_node = returning_target
                else: returning_node = self._to_snippet(n)

                self.kb.functions._add_fakeret_to(src_function.addr, src_node, returning_node, confirmed=True,
                                                  to_outside=to_outside
                                                  )

        elif jumpkind == 'Ijk_Boring':

            # convert src_addr and dst_addr to CodeNodes
            n = self.get_any_node(src_addr)
            if n is None: src_node = src_addr
            else: src_node = self._to_snippet(n)

            n = self.get_any_node(dst_addr)
            if n is None: dst_node = dst_addr
            else: dst_node = self._to_snippet(n)

            # pre-check: if source and destination do not belong to the same section, it must be jumping to another
            # function
            if not self._addrs_belong_to_same_section(src_addr, dst_addr):
                _ = self._addr_to_function(dst_addr, blockaddr_to_function, known_functions)

            # is it a jump to another function?
            if dst_addr in known_functions or (
                dst_addr in blockaddr_to_function and blockaddr_to_function[dst_addr] is not src_function
            ):
                # yes it is
                dst_function_addr = blockaddr_to_function[dst_addr].addr if dst_addr in blockaddr_to_function else \
                    dst_addr

                self.kb.functions._add_outside_transition_to(src_function.addr, src_node, dst_node,
                                                             to_function_addr=dst_function_addr
                                                             )

                _ = self._addr_to_function(dst_addr, blockaddr_to_function, known_functions)
            else:
                # no it's not
                # add the transition code

                if dst_addr not in blockaddr_to_function:
                    blockaddr_to_function[dst_addr] = src_function

                self.kb.functions._add_transition_to(src_function.addr, src_node, dst_node, ins_addr=ins_addr,
                                                     stmt_idx=stmt_idx
                                                     )

        elif jumpkind == 'Ijk_FakeRet':

            # convert src_addr and dst_addr to CodeNodes
            n = self.get_any_node(src_addr)
            if n is None: src_node = src_addr
            else: src_node = self._to_snippet(n)

            n = self.get_any_node(dst_addr)
            if n is None: dst_node = dst_addr
            else: dst_node = self._to_snippet(n)

            if dst_addr not in blockaddr_to_function:
                if dst_addr not in known_functions:
                    blockaddr_to_function[dst_addr] = src_function
                    target_function = src_function
                else:
                    target_function = self._addr_to_function(dst_addr, blockaddr_to_function, known_functions)
            else:
                target_function = blockaddr_to_function[dst_addr]

            to_outside = not target_function is src_function

            self.kb.functions._add_fakeret_to(src_function.addr, src_node, dst_node, confirmed=True,
                                              to_outside=to_outside, to_function_addr=target_function.addr
                                              )

        else:
            l.debug('Ignored jumpkind %s', jumpkind)

    #
    # Other functions
    #

    @staticmethod
    def _is_noop_block(vex_block):
        """
        Check if the block is a no-op block by checking VEX statements.

        :param vex_block: The VEX block instance.
        :return: True if the entire block is a single-byte or multi-byte nop instruction, False otherwise.
        :rtype: bool
        """

        # the block is a noop block if it only has IMark statements

        if all((type(stmt) is pyvex.IRStmt.IMark) for stmt in vex_block.statements):
            return True
        return False


    @staticmethod
    def _is_noop_insn(insn):
        """
        Check if the instruction does nothing.

        :param insn: The capstone insn object.
        :return: True if the instruction does no-op, False otherwise.
        """

        if insn.insn_name() == 'nop':
            # nops
            return True
        if insn.insn_name() == 'lea':
            # lea reg, [reg + 0]
            op0, op1 = insn.operands
            if op0.type == 1 and op1.type == 3:
                # reg and mem
                if op0.reg == op1.mem.base and op1.mem.index == 0 and op1.mem.disp == 0:
                    return True

        # add more types of no-op instructions here :-)

        return False

    @classmethod
    def _get_nop_length(cls, insns):
        """
        Calculate the total size of leading nop instructions.

        :param insns: A list of capstone insn objects.
        :return: Number of bytes of leading nop instructions.
        :rtype: int
        """

        nop_length = 0

        if insns and cls._is_noop_insn(insns[0]):
            # see where those nop instructions terminate
            for insn in insns:
                if cls._is_noop_insn(insn):
                    nop_length += insn.size
                else:
                    break

        return nop_length

    @staticmethod
    def _one_fakeret_node(all_edges):
        """
        Pick the first Ijk_FakeRet edge from all_edges, and return the destination node.

        :param list all_edges: A list of networkx.Graph edges with data.
        :return:               The first FakeRet node, or None if nothing is found.
        :rtype:                CFGNode or None
        """

        for src, dst, data in all_edges:
            if data.get('jumpkind', None) == 'Ijk_FakeRet':
                return dst
        return None



import logging

import claripy
import simuvex
from ..analysis import Analysis, register_analysis

l = logging.getLogger('angr.analyses.congruency_check')
#l.setLevel(logging.DEBUG)

class CongruencyCheck(Analysis):
	"""
	This is an analysis to ensure that angr executes things identically with different execution backends (i.e., unicorn vs vex).
	"""

	def __init__(self, throw=False):
		"""
		Initializes a CongruencyCheck analysis.

		:param throw: whether to raise an exception if an incongruency is found.
		"""
		self._throw = throw
		self.pg = None
		self.prev_pg = None

	def set_state_options(self, left_add_options=None, left_remove_options=None, right_add_options=None, right_remove_options=None):
		"""
		Checks that the specified state options result in the same states over the next `depth` states.
		"""
		s_right = self.project.factory.full_init_state(
			add_options=right_add_options, remove_options=right_remove_options
		)
		s_left = self.project.factory.full_init_state(
			add_options=left_add_options, remove_options=left_remove_options
		)

		return self.set_states(s_left, s_right)

	def set_states(self, left_state, right_state):
		"""
		Checks that the specified states stay the same over the next `depth` states.
		"""
		p_right = self.project.factory.path(right_state)
		p_left = self.project.factory.path(left_state)

		return self.set_paths(p_left, p_right)

	def set_paths(self, left_path, right_path):
		"""
		Checks that the specified paths stay the same over the next `depth` states.
		"""
		pg = self.project.factory.path_group(right_path)
		pg.stash(to_stash='right')
		pg.active.append(left_path)
		pg.stash(to_stash='left')
		pg.stash(to_stash='stashed_left')
		pg.stash(to_stash='stashed_right')

		return self.set_path_group(pg)

	def set_path_group(self, pg):
		self.pg = pg
		return self

	@staticmethod
	def _sync_steps(pg, max_steps=None):
		l.debug("Sync-stepping pathgroup...")
		l.debug(
			"... left width: %s, right width: %s",
			pg.left[0].weighted_length if len(pg.left) > 0 else None,
			pg.right[0].weighted_length if len(pg.right) > 0 else None,
		)

		if len(pg.errored) != 0 and (len(pg.left) == 0 or len(pg.right) == 0):
			l.debug("... looks like a path errored")
			return pg
		if len(pg.left) == 0 and len(pg.right) != 0:
			l.debug("... left is deadended; stepping right %s times", max_steps)
			npg = pg.step(stash='right', n=max_steps)
		elif len(pg.right) == 0 and len(pg.left) != 0:
			l.debug("... right is deadended; stepping left %s times", max_steps)
			npg = pg.step(stash='left', n=max_steps)
		elif len(pg.right) == 0 and len(pg.left) == 0:
			l.debug("... both deadended.")
			return pg
		elif pg.left[0].weighted_length == pg.right[0].weighted_length:
			l.debug("... synced")
			return pg
		elif pg.left[0].weighted_length < pg.right[0].weighted_length:
			l.debug("... right is ahead; stepping left up to %s times", max_steps)
			npg = pg.step(
				stash='left',
				until=lambda lpg: lpg.left[0].weighted_length >= pg.right[0].weighted_length,
				n=max_steps
			)
		elif pg.right[0].weighted_length < pg.left[0].weighted_length:
			l.debug("... left is ahead; stepping right up to %s times", max_steps)
			npg = pg.step(
				stash='right',
				until=lambda lpg: lpg.right[0].weighted_length >= pg.left[0].weighted_length,
				n=max_steps
			)

		return CongruencyCheck._sync_steps(npg)

	def _validate_incongruency(self):
		"""
		Checks that a detected incongruency is not caused by translation backends having a different
		idea of what constitutes a basic block.
		"""

		ot = self._throw

		try:
			self._throw = False
			l.debug("Validating incongruency.")

			if ("UNICORN" in self.pg.right[0].state.options) ^ ("UNICORN" in self.pg.left[0].state.options):
				if "UNICORN" in self.pg.right[0].state.options:
					unicorn_stash = 'right'
					normal_stash = 'left'
				else:
					unicorn_stash = 'left'
					normal_stash = 'right'

				unicorn_path = self.pg.stashes[unicorn_stash][0]
				normal_path = self.pg.stashes[normal_stash][0]

				if unicorn_path.state.arch.name in ("X86", "AMD64"):
					# unicorn "falls behind" on loop and rep instructions, since
					# it sees them as ending a basic block. Here, we will
					# step the unicorn until it's caught up
					npg = self.project.factory.path_group(unicorn_path)
					npg.explore(find=lambda p: p.addr == normal_path.addr, n=200)
					if len(npg.found) == 0:
						l.debug("Validator failed to sync paths.")
						return True

					new_unicorn = npg.found[0]
					delta = new_unicorn.weighted_length - normal_path.weighted_length
					normal_path.extra_length += delta
					new_normal = normal_path
				elif unicorn_path.state.arch.name == "MIPS32":
					# unicorn gets ahead here, because VEX falls behind for unknown reasons
					# for example, this block:
					#
					# 0x1016f20:      lui     $gp, 0x17
					# 0x1016f24:      addiu   $gp, $gp, -0x35c0
					# 0x1016f28:      addu    $gp, $gp, $t9
					# 0x1016f2c:      addiu   $sp, $sp, -0x28
					# 0x1016f30:      sw      $ra, 0x24($sp)
					# 0x1016f34:      sw      $s0, 0x20($sp)
					# 0x1016f38:      sw      $gp, 0x10($sp)
					# 0x1016f3c:      lw      $v0, -0x6cf0($gp)
					# 0x1016f40:      move    $at, $at
					npg = self.project.factory.path_group(normal_path)
					npg.explore(find=lambda p: p.addr == unicorn_path.addr, n=200)
					if len(npg.found) == 0:
						l.debug("Validator failed to sync paths.")
						return True

					new_normal = npg.found[0]
					delta = new_normal.weighted_length - unicorn_path.weighted_length
					unicorn_path.extra_length += delta
					new_unicorn = unicorn_path
				else:
					l.debug("Dunno!")
					return True

				if self.compare_paths(new_unicorn, new_normal):
					l.debug("Divergence accounted for by unicorn.")
					self.pg.stashes[unicorn_stash][0] = new_unicorn
					self.pg.stashes[normal_stash][0] = new_normal
					return False
				else:
					l.warning("Divergence unaccounted for by unicorn.")
					return True
			else:
				# no idea
				l.warning("Divergence unaccounted for.")
				return True
		finally:
			self._throw = ot

	def _report_incongruency(self, *args):
		l.warning(*args)
		if self._throw:
			raise AngrIncongruencyError(*args)

	def run(self, depth=None):
		"""
		Checks that the paths in the specified path group stay the same over the next
		`depth` bytes.

		The path group should have a "left" and a "right" stash, each with a single
		path.
		"""
		#pg_history = [ ]
		if len(self.pg.right) != 1 or len(self.pg.left) != 1:
			self._report_incongruency("Single path in pg.left and pg.right required.")
			return False

		if "UNICORN" in self.pg.one_right.state.options and depth is not None:
			self.pg.one_right.state.unicorn.max_steps = depth

		if "UNICORN" in self.pg.one_left.state.options and depth is not None:
			self.pg.one_left.state.unicorn.max_steps = depth

		l.debug("Performing initial path comparison.")
		if not self.compare_paths(self.pg.left[0], self.pg.right[0]):
			self._report_incongruency("Initial path comparison check failed.")
			return False

		while len(self.pg.left) > 0 and len(self.pg.right) > 0:
			if depth is not None:
				self._update_progress(100. * float(self.pg.one_left.weighted_length) / depth)

			if len(self.pg.deadended) != 0:
				self._report_incongruency("Unexpected deadended paths before step.")
				return False
			if len(self.pg.right) == 0 and len(self.pg.left) == 0:
				l.debug("All done!")
				return True
			if len(self.pg.right) != 1 or len(self.pg.left) != 1:
				self._report_incongruency("Different numbers of paths in left and right stash..")
				return False

			# do a step
			l.debug(
				"Stepping right path with weighted length %d/%s",
				self.pg.right[0].weighted_length,
				depth
			)
			self.prev_pg = self.pg.copy() #pylint:disable=unused-variable
			self.pg.step(stash='right')
			CongruencyCheck._sync_steps(self.pg)

			if len(self.pg.errored) != 0:
				self._report_incongruency("Unexpected errored paths.")
				return False

			try:
				if not self.compare_path_group(self.pg) and self._validate_incongruency():
					self._report_incongruency("Path group comparison failed.")
					return False
			except AngrIncongruencyError:
				if self._validate_incongruency():
					raise

			if depth is not None:
				self.pg.drop(stash='left', filter_func=lambda p: p.weighted_length >= depth)
				self.pg.drop(stash='right', filter_func=lambda p: p.weighted_length >= depth)

			self.pg.right.sort(key=lambda p: p.addr)
			self.pg.left.sort(key=lambda p: p.addr)
			self.pg.stashed_right[:] = self.pg.stashed_right[::-1]
			self.pg.stashed_left[:] = self.pg.stashed_left[::-1]
			self.pg.move('stashed_right', 'right')
			self.pg.move('stashed_left', 'left')

			if len(self.pg.left) > 1:
				self.pg.split(from_stash='left', limit=1, to_stash='stashed_left')
				self.pg.split(from_stash='right', limit=1, to_stash='stashed_right')

	def compare_path_group(self, pg):
		if len(pg.left) != len(pg.right):
			self._report_incongruency("Number of left and right paths differ.")
			return False
		if len(pg.deadended) % 2 != 0:
			self._report_incongruency("Odd number of deadended paths after step.")
			return False
		pg.drop(stash='deadended')

		if len(pg.left) == 0 and len(pg.right) == 0:
			return True

		# make sure the paths are the same
		for pl,pr in zip(sorted(pg.left, key=lambda p: p.addr), sorted(pg.right, key=lambda p: p.addr)):
			if not self.compare_paths(pl, pr):
				self._report_incongruency("Differing paths.")
				return False

		return True

	def compare_states(self, sl, sr):
		"""
		Compares two states for similarity.
		"""
		joint_solver = claripy.Solver()

		# make sure the canonicalized constraints are the same
		n_map, n_counter, n_canon_constraint = claripy.And(*sr.se.constraints).canonicalize() #pylint:disable=no-member
		u_map, u_counter, u_canon_constraint = claripy.And(*sl.se.constraints).canonicalize() #pylint:disable=no-member
		n_canoner_constraint = sr.se.simplify(n_canon_constraint)
		u_canoner_constraint = sl.se.simplify(u_canon_constraint)
		joint_solver.add((n_canoner_constraint, u_canoner_constraint))
		if n_canoner_constraint is not u_canoner_constraint:
			self._report_incongruency("Different constraints!")
			return False

		# get the differences in registers and memory
		mem_diff = sr.memory.changed_bytes(sl.memory)
		reg_diff = sr.registers.changed_bytes(sl.registers)

		# this is only for unicorn
		if "UNICORN" in sl.options | sr.options:
			if sl.arch.name == "X86":
				reg_diff -= set(range(40, 52)) #ignore cc psuedoregisters
				reg_diff -= set(range(320, 324)) #some other VEX weirdness
				reg_diff -= set(range(340, 344)) #ip_at_syscall
			elif sl.arch.name == "AMD64":
				reg_diff -= set(range(144, 168)) #ignore cc psuedoregisters

		# make sure the differences in registers and memory are actually just renamed
		# versions of the same ASTs
		for diffs,(um,nm) in (
			(reg_diff, (sl.registers, sr.registers)),
			(mem_diff, (sl.memory, sr.memory)),
		):
			for i in diffs:
				bn = nm.load(i, 1)
				bu = um.load(i, 1)

				bnc = bn.canonicalize(var_map=n_map, counter=n_counter)[-1]
				buc = bu.canonicalize(var_map=u_map, counter=u_counter)[-1]

				if bnc is not buc:
					self._report_incongruency("Different memory or registers (index %d, values %r and %r)!", i, bn, bu)
					return False

		# make sure the flags are the same
		if sl.arch.name in ("AMD64", "X86", "ARM", "AARCH64"):
			n_flags = simuvex.engines.vex.ccall._get_flags(sr)[0].canonicalize(var_map=n_map, counter=n_counter)[-1]
			u_flags = simuvex.engines.vex.ccall._get_flags(sl)[0].canonicalize(var_map=u_map, counter=u_counter)[-1]
			if n_flags is not u_flags and sl.se.simplify(n_flags) is not sr.se.simplify(u_flags):
				self._report_incongruency("Different flags!")
				return False

		return True

	def compare_paths(self, pl, pr):
		l.debug("Comparing paths...")
		if not self.compare_states(pl.state, pr.state):
			self._report_incongruency("Failed state similarity check!")
			return False

		if pr.weighted_length != pl.weighted_length:
			self._report_incongruency("Different weights!")
			return False

		if pl.addr != pr.addr:
			self._report_incongruency("Different addresses!")
			return False

		return True

from ..errors import AngrIncongruencyError
register_analysis(CongruencyCheck, 'CongruencyCheck')




import cle

from . import MipsElfFastResolver
from . import X86ElfPicPltResolver


DEFAULT_RESOLVERS = {
    'X86': {
        cle.MetaELF: [ X86ElfPicPltResolver, ],
    },
    'MIPS32': {
        cle.MetaELF: [ MipsElfFastResolver, ],
    }
}


def default_indirect_jump_resolvers(arch, obj, project=None):
    arch_specific = DEFAULT_RESOLVERS.get(arch.name, { })
    resolvers = [ ]
    for k, lst in arch_specific.iteritems():
        if isinstance(obj, k):
            resolvers = lst
            break

    return [ r(project) for r in resolvers ]



import logging

from ..analysis import Analysis, register_analysis
from networkx import DiGraph
from copy import copy

l = logging.getLogger(name="angr.analyses.dfg")

class DFG(Analysis):
    def __init__(self, cfg=None, annocfg=None):
        """
        Build a Data Flow Grah (DFG) for every basic block of a CFG

        The DFGs are available in the dict self.dfgs where the key
        is a basic block addr and the value a DFG.

        :param cfg: A CFG used to get all the basic blocks
        :param annocfg: An AnnotatedCFG built from a backward slice used to only build the DFG on the whitelisted statements
        """
        if cfg is None:
            self._cfg = self.project.analyses.CFGAccurate()
        else:
            self._cfg = cfg
        self._annocfg = annocfg

        self.dfgs = self._construct()

    def _need_to_ignore(self, addr, stmt, stmt_idx):
        if self._annocfg is not None:
            whitelist = self._annocfg.get_whitelisted_statements(addr)
            if whitelist is False or (whitelist is not None and stmt_idx not in whitelist):
                return True
        if stmt.tag == 'Ist_IMark' or stmt.tag == 'Ist_AbiHint' or stmt.tag == 'Ist_Exit':
            return True
        elif stmt.tag == 'Ist_Put':
            arch = self.project.arch
            if stmt.offset in arch.register_names:
                if stmt.offset == arch.ip_offset:
                    return True
        return False

    def _construct(self):
        """
        We want to build the type of DFG that's used in "Automated Ident. of Crypto
        Primitives in Binary Code with Data Flow Graph Isomorphisms." Unlike that
        paper, however, we're building it on Vex IR instead of assembly instructions.
        """
        cfg = self._cfg
        p = self.project
        dfgs = {}
        l.debug("Building Vex DFG...")

        for node in cfg.nodes():
            try:
                if node.simprocedure_name == None:
                    irsb = p.factory.block(node.addr).vex
                else:
                    l.debug("Cannot process SimProcedures, ignoring %s" % node.simprocedure_name)
                    continue
            except Exception as e:
                l.debug(e)
                continue
            tmpsnodes = {}
            storesnodes = {}
            putsnodes = {}
            statements = irsb.statements
            dfg = DiGraph()

            for stmt_idx, stmt in enumerate(statements):
                # We want to skip over certain types, such as Imarks
                if self._need_to_ignore(node.addr, stmt, stmt_idx):
                    continue

                # break statement down into sub-expressions
                exprs = stmt.expressions
                stmt_node = stmt
                dfg.add_node(stmt)

                if stmt.tag == 'Ist_WrTmp':
                    tmpsnodes[stmt.tmp] = stmt_node
                    if exprs[0].tag == 'Iex_Binop':
                        if exprs[1].tag == 'Iex_RdTmp':
                            dfg.add_edge(tmpsnodes[exprs[1].tmp], stmt_node)
                        else:
                            dfg.add_edge(exprs[1], stmt_node)
                        if exprs[2].tag == 'Iex_RdTmp':
                            dfg.add_edge(tmpsnodes[exprs[2].tmp], stmt_node)
                        else:
                            dfg.add_edge(exprs[2], stmt_node)

                    elif exprs[0].tag == 'Iex_Unop':
                        dfg.remove_node(stmt_node)
                        if exprs[1].tag == 'Iex_RdTmp':
                            tmpsnodes[stmt.tmp] = copy(tmpsnodes[exprs[1].tmp])
                            tmpsnodes[stmt.tmp].tmp = stmt.tmp
                        else:
                            tmpsnodes[stmt.tmp] = exprs[1]

                    elif exprs[0].tag == 'Iex_RdTmp':
                        tmpsnodes[stmt.tmp] = copy(tmpsnodes[exprs[0].tmp])
                        tmpsnodes[stmt.tmp].tmp = stmt.tmp

                    elif exprs[0].tag == 'Iex_Get':
                        if putsnodes.has_key(exprs[0].offset):
                            dfg.add_edge(putsnodes[exprs[0].offset], stmt_node)
                        if len(exprs) > 1 and exprs[1].tag == "Iex_RdTmp":
                            dfg.add_edge(tmpsnodes[exprs[1].tmp], stmt_node)
                        elif len(exprs) > 1:
                            dfg.add_edge(exprs[1], stmt_node)

                    elif exprs[0].tag == 'Iex_Load':
                        if exprs[1].tag == 'Iex_RdTmp':
                            dfg.add_edge(tmpsnodes[exprs[1].tmp], stmt_node)
                        else:
                            dfg.add_edge(exprs[1], stmt_node)

                    else:
                        # Take a guess by assuming exprs[0] is the op and any other expressions are args
                        for e in exprs[1:]:
                            if e.tag == 'Iex_RdTmp':
                                dfg.add_edge(tmpsnodes[e.tmp], stmt_node)
                            else:
                                dfg.add_edge(e, stmt_node)

                elif stmt.tag == 'Ist_Store':
                    if exprs[0].tag == 'Iex_RdTmp':
                        dfg.add_edge(tmpsnodes[exprs[0].tmp], stmt_node)

                    elif exprs[0].tag == 'Iex_Const':
                        dfg.add_edge(exprs[0], stmt_node)

                    if exprs[1].tag == 'Iex_RdTmp':
                        dfg.add_edge(tmpsnodes[exprs[1].tmp], stmt_node)
                    else:
                        dfg.add_edge(exprs[1], stmt_node)

                elif stmt.tag == 'Ist_Put':
                    if exprs[0].tag == 'Iex_RdTmp':
                        dfg.add_edge(tmpsnodes[exprs[0].tmp], stmt_node)
                    elif exprs[0].tag == 'Iex_Const':
                        dfg.add_edge(exprs[0], stmt_node)
                    putsnodes[stmt.offset] = stmt_node

                elif stmt.tag == 'Ist_Exit':
                    if exprs[0].tag == 'Iex_RdTmp':
                        dfg.add_edge(tmpsnodes[exprs[0].tmp], stmt_node)

                elif stmt.tag == 'Ist_Dirty':
                    tmpsnodes[stmt.tmp] = stmt_node
                elif stmt.tag == 'Ist_CAS':
                    tmpsnodes[stmt.oldLo] = stmt_node

                else:
                    for e in stmt.expressions:
                        if e.tag == 'Iex_RdTmp':
                            dfg.add_edge(tmpsnodes[e.tmp], stmt_node)
                        else:
                            dfg.add_edge(e, stmt_node)

            for vtx in dfg.nodes():
                if dfg.degree(vtx) == 0:
                    dfg.remove_node(vtx)

            if dfg.size() > 0:
                dfgs[node.addr] = dfg
        return dfgs

register_analysis(DFG, 'DFG')



import inspect
import logging
import itertools
from collections import defaultdict

import ana
import simuvex
import claripy
import mulpyplexer

l = logging.getLogger('angr.path_group')

class PathGroup(ana.Storable):
    """
    Path groups are the future.

    Path groups allow you to wrangle multiple paths in a slick way. Paths are organized into "stashes", which you can
    step forward, filter, merge, and move around as you wish. This allows you to, for example, step two different
    stashes of paths at different rates, then merge them together.

    Stashes can be accessed as attributes (i.e. pg.active). A mulpyplexed stash can be retrieved by prepending the name
    with `mp_` (e.g., `pg.mp_active`).

    Note that you shouldn't usually be constructing path groups directly - there is a convenient shortcuts for
    creating path groups in ``Project.factory``: see :class:`angr.factory.AngrObjectFactory`.

    Multithreading your search can be useful in constraint-solving-intensive paths. Indeed, Python cannot multithread
    due to its GIL, but z3, written in C, can.

    The most important methods you should look at are ``step``, ``explore``, and ``use_tech``.
    """

    ALL = '_ALL'
    DROP = '_DROP'

    def __init__(self, project, active_paths=None, stashes=None, hierarchy=None, veritesting=None,
                 veritesting_options=None, immutable=None, resilience=None, save_unconstrained=None,
                 save_unsat=None, threads=None):
        """
        :param project:         A Project instance.
        :type  project:         angr.project.Project

        The following parameters are optional.

        :param active_paths:    Active paths to seed the "active" stash with.
        :param stashes:         A dictionary to use as the stash store.
        :param hierarchy:       A PathHierarchy object to use to track path reachability.
        :param immutable:       If True, all operations will return a new PathGroup. Otherwise (default), all operations
                                will modify the PathGroup (and return it, for consistency and chaining).
        :param threads:         the number of worker threads to concurrently analyze states (useful in z3-intensive paths).
        """
        self._project = project
        self._hierarchy = PathHierarchy() if hierarchy is None else hierarchy
        self._immutable = False if immutable is None else immutable
        self._resilience = False if resilience is None else resilience

        # public options
        self.save_unconstrained = False if save_unconstrained is None else save_unconstrained
        self.save_unsat = False if save_unsat is None else save_unsat

        # techniques
        self._hooks_step = []
        self._hooks_step_path = []
        self._hooks_filter = []
        self._hooks_complete = []

        if threads is not None:
            self.use_technique(exploration_techniques.Threading(threads))
        if veritesting:
            self.use_technique(exploration_techniques.Veritesting(
                **({} if veritesting_options is None else veritesting_options)
            ))

        self.stashes = self._make_stashes_dict(active=active_paths) if stashes is None else stashes

    #
    # Pickling
    #

    def _ana_getstate(self):
        self.prune()
        s = dict(self.__dict__)
        if self._hierarchy is not False:
            s['_hierarchy'] = None
        return s

    def _ana_setstate(self, s):
        self.__dict__.update(s)
        if self._hierarchy is None:
            self._hierarchy = PathHierarchy()

    #
    # Util functions
    #

    def copy(self, stashes=None):
        stashes = stashes if stashes is not None else self._copy_stashes(immutable=True)
        out = PathGroup(self._project, stashes=stashes, hierarchy=self._hierarchy, immutable=self._immutable, resilience=self._resilience, save_unconstrained=self.save_unconstrained, save_unsat=self.save_unsat)
        out._hooks_step = list(self._hooks_step)
        out._hooks_step_path = list(self._hooks_step_path)
        out._hooks_filter = list(self._hooks_filter)
        out._hooks_complete = list(self._hooks_complete)
        return out

    @staticmethod
    def _make_stashes_dict(active=None, unconstrained=None, unsat=None, pruned=None, errored=None, deadended=None,
                           **kwargs):

        always_present = {'active': active or [],
                          'unconstrained': unconstrained or [],
                          'unsat': unsat or [],
                          'pruned': pruned or [],
                          'errored': errored or [],
                          'deadended': deadended or []
                          }

        result = defaultdict(list, always_present, **kwargs)
        return result

    def _copy_stashes(self, immutable=None):
        """
        Returns a copy of the stashes (if immutable) or the stashes themselves (if not immutable). Used to abstract away
        immutability.
        """
        if self._immutable if immutable is None else immutable:
            result = self._make_stashes_dict(**{k: list(v) for k, v in self.stashes.items()})
        else:
            result = self.stashes

        return result

    def _copy_paths(self, paths):
        """
        Returns a copy of a list of paths (if immutable) or the paths themselves (if not immutable). Used to abstract
        away immutability.
        """
        if self._immutable:
            return [ p.copy() for p in paths ]
        else:
            return paths

    def _successor(self, new_stashes):
        """
        Creates a new PathGroup with the provided stashes (if immutable), or sets the stashes (if not immutable). Used
        to abstract away immutability.

        :returns:   A PathGroup.
        """
        if self.DROP in new_stashes:
            del new_stashes[self.DROP]

        if not self._immutable:
            self.stashes = defaultdict(list, new_stashes.iteritems())
            return self
        else:
            return self.copy(stashes=new_stashes)


    @staticmethod
    def _filter_paths(filter_func, paths):
        """
        Filters a sequence of paths according to a filter_func.

        :param filter_func: The filter function. Should take a path as input and return a boolean.
        :param paths:       A sequence of paths.

        :returns:           A tuple, with the first element the matching paths and the second element the non-matching
                            paths.
        """
        if filter_func is None:
            return paths, []        # does this condition actually matter

        l.debug("Filtering %d paths", len(paths))
        match = [ ]
        nomatch = [ ]

        for p in paths:
            if filter_func(p):
                l.debug("... path %s matched!", p)
                match.append(p)
            else:
                l.debug("... path %s didn't match!", p)
                nomatch.append(p)

        l.debug("... returning %d matches and %d non-matches", len(match), len(nomatch))
        return match, nomatch

    def _one_path_step(self, a, check_func=None, successor_func=None, **kwargs):
        """
        Internal function to step a single path forward.

        :param a:               The path.
        :param check_func:      A function to check the path for an error state.
        :param successor_func:  A function to run on the path instead of doing a.step().

        :returns:               A dict mapping stash names to path lists
        """

        # we keep a strong reference here since the hierarchy handles trimming it
        if self._hierarchy:
            kwargs["strong_reference"] = True

        for hook in self._hooks_step_path:
            # FIXME hack to handle some hooks not expecting strong_reference
            argspec = inspect.getargspec(hook)
            if argspec.keywords is None and "strong_reference" not in argspec.args:
                del kwargs["strong_reference"]

            out = hook(a, **kwargs)
            if out is not None:
                if isinstance(out, tuple):
                    l.warning('step_path returning a tuple has been deprecated! Please return a dict of stashes instead.')
                    a, unconst, unsat, p, e = out
                    out = {'active': a, 'unconstrained': unconst, 'unsat': unsat, 'pruned': p, 'errored': e}
                return self._make_stashes_dict(**out)

        if (check_func is not None and check_func(a)) or (check_func is None and a.errored):
            # This path has error(s)!
            if hasattr(a, "error") and isinstance(a.error, PathUnreachableError):
                return self._make_stashes_dict(pruned=[a])
            else:
                if self._hierarchy:
                    self._hierarchy.unreachable_path(a)
                    self._hierarchy.simplify()
                return self._make_stashes_dict(errored=[a])
        else:
            try:
                if successor_func is not None:
                    successors = successor_func(a)
                else:
                    successors = a.step(**kwargs)
                return self._make_stashes_dict(active=successors,
                                               unconstrained=a.unconstrained_successors,
                                               unsat=a.unsat_successors)
            except (AngrError, simuvex.SimError, claripy.ClaripyError):
                if not self._resilience:
                    raise
                else:
                    l.warning("PathGroup resilience squashed an exception", exc_info=True)
                    return self._make_stashes_dict(errored=[a])

    def _record_step_results(self, new_stashes, new_active, a, successor_stashes):
        """
        Take a whole bunch of intermediate values and smushes them together

        :param new_stashes:         The dict of stashes that will be modified by this operation
        :param new_active:          One of the lists in new_stashes that is being actively ticked
        :param a:                   The path that just got ticked
        :param successor_stashes:   A dict of the stashs and their paths that were created by stepping the path a.
        """

        if self._hierarchy:
            for p in successor_stashes.get('active', []):
                self._hierarchy.add_path(p)
            self._hierarchy.simplify()

        if len(self._hooks_filter) == 0:
            new_active.extend(successor_stashes['active'])
        else:
            for path in successor_stashes['active']:
                self._apply_filter_hooks(path,new_stashes,new_active)

        if self.save_unconstrained:
            new_stashes['unconstrained'] += successor_stashes['unconstrained']
        if self.save_unsat:
            new_stashes['unsat'] += successor_stashes['unsat']
        new_stashes['pruned'] += successor_stashes['pruned']
        new_stashes['errored'] += successor_stashes['errored']

        if a not in successor_stashes['pruned'] \
                and a not in successor_stashes['errored'] \
                and len(successor_stashes['active']) == 0:
            new_stashes['deadended'].append(a)

    def _apply_filter_hooks(self,path,new_stashes,new_active):

        for hook in self._hooks_filter:
            goto = hook(path)
            if goto is None:
                continue
            if type(goto) is tuple:
                goto, path = goto

            if goto in new_stashes:
                new_stashes[goto].append(path)
                break
            else:
                new_stashes[goto] = [path]
                break
        else:
            new_active.append(path)

        return new_active

    def _one_step(self, stash, selector_func=None, successor_func=None, check_func=None, **kwargs):
        """
        Takes a single step in a given stash.

        :param stash:           The name of the stash.
        :param successor_func:  If provided, this function is called with the path as its only argument. It should
                                return the path's successors. If this is None, path.successors is used, instead.
        :param selector_func:   If provided, should be a lambda that takes a Path and returns a boolean. If True, the
                                path will be stepped. Otherwise, it will be kept as-is.
        :param check_func:      If provided, this function will be called to decide whether the current path is errored
                                or not. Path.errored will not be called anymore.

        :returns:               The successor PathGroup.
        :rtype:                 PathGroup
        """
        if len(self._hooks_step) != 0:
            # hooking step is a bit of an ordeal, how are you supposed to compose stepping operations?
            # the answer is that you nest them - any stepping hook must eventually call step itself,
            # at which point it calls the next hook, and so on, until we fall through to the
            # basic stepping operation.
            hook = self._hooks_step.pop()
            pg = self.copy() if self._immutable else self
            pg._immutable = False       # this is a performance consideration
            out = hook(pg, stash, selector_func=selector_func, successor_func=successor_func, check_func=check_func, **kwargs)
            out._immutable = self._immutable
            self._hooks_step.append(hook)
            if out is not self:
                out._hooks_step.append(hook)
            return out

        new_stashes = self._copy_stashes()

        if selector_func is None:
            new_active = []
            to_tick = list(self.stashes[stash])
        else:
            new_active = []
            to_tick = []
            for a in self.stashes[stash]:
                if selector_func(a):
                    to_tick.append(a)
                else:
                    new_active.append(a)

        for a in to_tick:
            result_stashes = self._one_path_step(a, successor_func=successor_func, check_func=check_func, **kwargs)
            self._record_step_results(new_stashes, new_active, a, result_stashes)

        new_stashes[stash] = new_active
        return self._successor(new_stashes)

    @staticmethod
    def _move(stashes, filter_func, src, dst):
        """
        Moves all stashes that match the filter_func from src to dst.

        :returns: A new stashes dictionary.
        """
        if dst == PathGroup.ALL:
            raise AngrPathGroupError("Can't handle '_ALL' as a target stash.")
        if src == PathGroup.DROP:
            raise AngrPathGroupError("Can't handle '_DROP' as a source stash.")

        if src == PathGroup.ALL:
            srces = [ a for a in stashes.keys() if a != dst ]
        else:
            srces = [ src ]

        matches = [ ]
        for f in srces:
            to_move, to_keep = PathGroup._filter_paths(filter_func, stashes[f])
            stashes[f] = to_keep
            matches.extend(to_move)

        if dst != PathGroup.DROP:
            if dst not in stashes:
                stashes[dst] = [ ]
            stashes[dst].extend(matches)
        return stashes

    def __repr__(self):
        s = "<PathGroup with "
        s += ', '.join(("%d %s" % (len(v),k)) for k,v in self.stashes.items() if len(v) != 0)
        s += ">"
        return s

    def mulpyplex(self, *stashes):
        """
        Mulpyplex across several stashes.

        :param stashes: the stashes to mulpyplex
        :return: a mulpyplexed list of paths from the stashes in question, in the specified order
        """

        return mulpyplexer.MP(list(itertools.chain.from_iterable(self.stashes[s] for s in stashes)))

    def __getattr__(self, k):
        if k == PathGroup.ALL:
            return [ p for p in itertools.chain.from_iterable(s for s in self.stashes.values()) ]
        elif k == 'mp_' + PathGroup.ALL:
            return mulpyplexer.MP([ p for p in itertools.chain.from_iterable(s for s in self.stashes.values()) ])
        elif k.startswith('mp_'):
            return mulpyplexer.MP(self.stashes[k[3:]])
        elif k.startswith('one_') and k[4:] in self.stashes:
            return self.stashes[k[4:]][0]
        elif k in self.stashes:
            return self.stashes[k]
        else:
            raise AttributeError(k)

    def __dir__(self):
        return sorted(set(
            self.__dict__.keys() +
            dir(super(PathGroup, self)) +
            dir(type(self)) +
            self.stashes.keys() +
            ['mp_'+k for k in self.stashes.keys()] +
            ['one_'+k for k in self.stashes.keys()]
        ))

    #
    # Interface
    #

    def apply(self, path_func=None, stash_func=None, stash=None):
        """
        Applies a given function to a given stash.

        :param path_func:   A function to apply to every path. Should take a path and return a path. The returned path
                            will take the place of the old path. If the function *doesn't* return a path, the old
                            path will be used. If the function returns a list of paths, they will replace the original
                            paths.
        :param stash_func:  A function to apply to the whole stash. Should take a list of paths and return a list of
                            paths. The resulting list will replace the stash.

                            If both path_func and stash_func are provided path_func is applied first, then stash_func
                            is applied on the results.

        :returns:           The resulting PathGroup.
        :rtype:             PathGroup
        """
        stash = 'active' if stash is None else stash

        new_stashes = self._copy_stashes()
        new_paths = new_stashes[stash]
        if path_func is not None:
            new_new_paths = [ ]
            for p in new_paths:
                np = path_func(p)
                if isinstance(np, Path):
                    new_new_paths.append(np)
                elif isinstance(np, (list, tuple, set)):
                    new_new_paths.extend(np)
                else:
                    new_new_paths.append(p)
            new_paths = new_new_paths
        if stash_func is not None:
            new_paths = stash_func(new_paths)

        new_stashes[stash] = new_paths
        return self._successor(new_stashes)

    def split(self, stash_splitter=None, stash_ranker=None, path_ranker=None, limit=None, from_stash=None, to_stash=None):
        """
        Split a stash of paths. The stash from_stash will be split into two stashes depending on the other options
        passed in. If to_stash is provided, the second stash will be written there.

        stash_splitter overrides stash_ranker, which in turn overrides path_ranker. If no functions are provided, the
        paths are simply split according to the limit.

        The sort done with path_ranker is ascending.

        :param stash_splitter:  A function that should take a list of paths and return a tuple of two lists (the two
                                resulting stashes).
        :param stash_ranker:    A function that should take a list of paths and return a sorted list of paths. This list
                                will then be split according to "limit".
        :param path_ranker:     An alternative to stash_splitter. Paths will be sorted with outputs of this function.
                                used as a key. The first "limit" of them will be kept, the rest split off.
        :param limit:           For use with path_ranker. The number of paths to keep. Default: 8
        :param from_stash:      The stash to split (default: 'active')
        :param to_stash:        The stash to write to (default: 'stashed')

        :returns:               The resulting PathGroup.
        :rtype:                 PathGroup
        """

        limit = 8 if limit is None else limit
        from_stash = 'active' if from_stash is None else from_stash
        to_stash = 'stashed' if to_stash is None else to_stash

        new_stashes = self._copy_stashes()
        old_paths = new_stashes[from_stash]

        if stash_splitter is not None:
            keep, split = stash_splitter(old_paths)
        elif stash_ranker is not None:
            ranked_paths = stash_ranker(old_paths)
            keep, split = ranked_paths[:limit], ranked_paths[limit:]
        elif path_ranker is not None:
            ranked_paths = sorted(old_paths, key=path_ranker)
            keep, split = ranked_paths[:limit], ranked_paths[limit:]
        else:
            keep, split = old_paths[:limit], old_paths[limit:]

        new_stashes[from_stash] = keep
        new_stashes[to_stash] = split if to_stash not in new_stashes else new_stashes[to_stash] + split
        return self._successor(new_stashes)

    def step(self, n=None, selector_func=None, step_func=None, stash=None,
             successor_func=None, until=None, check_func=None, **kwargs):
        """
        Step a stash of paths forward, i.e. run :meth:`angr.path.Path.step` on each of the individual paths in a stash
        and categorize the successors appropriately.

        The parameters to this function allow you to control everything about the stepping and categorization process.

        :param stash:           The name of the stash to step (default: 'active')
        :param n:               The number of times to step (default: 1 if "until" is not provided)
        :param selector_func:   If provided, should be a function that takes a Path and returns a boolean. If True, the
                                path will be stepped. Otherwise, it will be kept as-is.
        :param step_func:       If provided, should be a function that takes a PathGroup and returns a PathGroup. Will
                                be called with the PathGroup at every step. Note that this function should not actually
                                perform any stepping - it is meant to be a maintenance function called after each step.
        :param successor_func:  If provided, should be a function that takes a path and return its successors.
                                Otherwise, Path.successors will be used.
        :param until:           If provided, should be a function that takes a PathGroup and returns True or False.
                                Stepping will terminate when it is True.
        :param check_func:      If provided, this function will be called to decide whether the current path is errored
                                or not. Otherwise, Path.errored will be used.

        Additionally, you can pass in any of the following keyword args for project.factory.sim_run:

        :param jumpkind:        The jumpkind of the previous exit
        :param addr:            An address to execute at instead of the state's ip.
        :param stmt_whitelist:  A list of stmt indexes to which to confine execution.
        :param last_stmt:       A statement index at which to stop execution.
        :param thumb:           Whether the block should be lifted in ARM's THUMB mode.
        :param backup_state:    A state to read bytes from instead of using project memory.
        :param opt_level:       The VEX optimization level to use.
        :param insn_bytes:      A string of bytes to use for the block instead of the project.
        :param size:            The maximum size of the block, in bytes.
        :param num_inst:        The maximum number of instructions.
        :param traceflags:      traceflags to be passed to VEX. Default: 0

        :returns:               The resulting PathGroup.
        :rtype:                 PathGroup
        """
        stash = 'active' if stash is None else stash
        n = n if n is not None else 1 if until is None else 100000
        pg = self

        # Check for found path in first block
        new_active = []
        for path in pg.stashes[stash]:
            self._apply_filter_hooks(path,pg.stashes,new_active)
        pg.stashes[stash] = new_active


        for i in range(n):
            l.debug("Round %d: stepping %s", i, pg)

            pg = pg._one_step(stash=stash, selector_func=selector_func, successor_func=successor_func, check_func=check_func, **kwargs)
            if step_func is not None:
                pg = step_func(pg)

            if len(pg.stashes[stash]) == 0:
                l.debug("Out of paths in stash %s", stash)
                break

            if until is not None and until(pg):
                l.debug("Until function returned true")
                break

        return pg

    def prune(self, filter_func=None, from_stash=None, to_stash=None):
        """
        Prune unsatisfiable paths from a stash.
        This function will move all unsatisfiable or errored paths in the given stash into a different stash.

        :param filter_func: Only prune paths that match this filter.
        :param from_stash:  Prune paths from this stash. (default: 'active')
        :param to_stash:    Put pruned paths in this stash. (default: 'pruned')

        :returns:           The resulting PathGroup.
        :rtype:             PathGroup
        """
        to_stash = 'pruned' if to_stash is None else to_stash
        from_stash = 'active' if from_stash is None else from_stash

        to_prune, new_active = self._filter_paths(filter_func, self.stashes[from_stash])
        new_stashes = self._copy_stashes()

        for p in to_prune:
            if p.errored or not p.state.satisfiable():
                if to_stash not in new_stashes:
                    new_stashes[to_stash] = [ ]
                new_stashes[to_stash].append(p)
                if self._hierarchy:
                    self._hierarchy.unreachable_path(p)
                    self._hierarchy.simplify()
            else:
                new_active.append(p)

        new_stashes[from_stash] = new_active
        return self._successor(new_stashes)

    def move(self, from_stash, to_stash, filter_func=None):
        """
        Move paths from one stash to another.

        :param from_stash:  Take matching paths from this stash.
        :param to_stash:    Put matching paths into this stash.
        :param filter_func: Stash paths that match this filter. Should be a function that takes a path and returns
                            True or False. Default: stash all paths

        :returns:           The resulting PathGroup.
        :rtype:             PathGroup
        """
        new_stashes = self._copy_stashes()
        self._move(new_stashes, filter_func, from_stash, to_stash)
        return self._successor(new_stashes)

    def stash(self, filter_func=None, from_stash=None, to_stash=None):
        """
        Stash some paths. This is an alias for move(), with defaults for the stashes.

        :param filter_func: Stash paths that match this filter. Should be a function. that takes a path and returns True
                            or False. (default: stash all paths)
        :param from_stash:  Take matching paths from this stash. (default: 'active')
        :param to_stash:    Put matching paths into this stash. (default: 'stashed')

        :returns:           The resulting PathGroup
        :rtype:             PathGroup
        """
        to_stash = 'stashed' if to_stash is None else to_stash
        from_stash = 'active' if from_stash is None else from_stash
        return self.move(from_stash, to_stash, filter_func=filter_func)

    def drop(self, filter_func=None, stash=None):
        """
        Drops paths from a stash. This is an alias for move(), with defaults for the stashes.

        :param filter_func: Drop paths that match this filter. Should be a function that takes a path and returns True
                            or False. (default: drop all paths)
        :param stash:       Drop matching paths from this stash. (default: 'active')

        :returns:           The resulting PathGroup
        :rtype:             PathGroup
        """
        stash = 'active' if stash is None else stash
        return self.move(stash, self.DROP, filter_func=filter_func)

    def unstash(self, filter_func=None, to_stash=None, from_stash=None):
        """
        Unstash some paths. This is an alias for move(), with defaults for the stashes.

        :param filter_func: Unstash paths that match this filter. Should be a function that takes a path and returns
                            True or False. (default: unstash all paths)
        :param from_stash:  take matching paths from this stash. (default: 'stashed')
        :param to_stash:    put matching paths into this stash. (default: 'active')

        :returns:            The resulting PathGroup.
        :rtype:             PathGroup
        """
        to_stash = 'active' if to_stash is None else to_stash
        from_stash = 'stashed' if from_stash is None else from_stash
        return self.move(from_stash, to_stash, filter_func=filter_func)

    def _merge_paths(self, paths):
        """
        Merges a list of paths.

        :param paths: the paths to merge
        :returns: the resulting path
        :rtype: Path
        """

        if self._hierarchy:
            optimal, common_history, others = self._hierarchy.most_mergeable(paths)
        else:
            optimal, common_history, others = paths, None, [ ]

        if len(optimal) < 2:
            raise AngrPathGroupError("unable to find merge candidates")
        o = optimal.pop()
        m = o.merge(optimal, common_history)
        if self._hierarchy:
            self._hierarchy.add_path(m)

        if len(others):
            others.append(m)
            return self._merge_paths(others)
        else:
            return m

    def merge(self, merge_func=None, stash=None):
        """
        Merge the states in a given stash.

        :param stash:       The stash (default: 'active')
        :param merge_func:  If provided, instead of using path.merge, call this function with the paths as the argument.
                            Should return the merged path.

        :returns:           The result PathGroup.
        :rtype:             PathGroup
        """
        stash = 'active' if stash is None else stash
        self.prune(from_stash=stash)
        to_merge = self.stashes[stash]
        not_to_merge = [ ]

        merge_groups = [ ]
        while len(to_merge) > 0:
            g, to_merge = self._filter_paths(lambda p: p.addr == to_merge[0].addr, to_merge)
            if len(g) <= 1:
                not_to_merge.extend(g)
            else:
                merge_groups.append(g)

        for g in merge_groups:
            try:
                m = self._merge_paths(g) if merge_func is None else merge_func(*g)
                not_to_merge.append(m)
            except simuvex.SimMergeError:
                l.warning("SimMergeError while merging %d paths", len(g), exc_info=True)
                not_to_merge.extend(g)

        new_stashes = self._copy_stashes()
        new_stashes[stash] = not_to_merge
        return self._successor(new_stashes)

    def use_technique(self, tech):
        """
        Use an exploration technique with this path group.
        Techniques can be found in :mod:`angr.exploration_techniques`.

        :param tech:       An ExplorationTechnique object that contains code to modify this path group's behavior
        """
        # this might be the best worst code I've ever written in my life
        tech.project = self._project
        self.remove_tech(tech)
        tech.setup(self)
        for hook in ['step_path', 'step', 'filter', 'complete']:
            hookfunc = getattr(tech, hook)
            if hookfunc.im_func is not getattr(exploration_techniques.ExplorationTechnique, hook).im_func:
                getattr(self, '_hooks_' + hook).append(hookfunc)

    def remove_tech(self, tech):
        for hook in ['step_path', 'step', 'filter', 'complete']:
            try:
                getattr(self, '_hooks_' + hook).remove(getattr(tech, hook))
            except ValueError:
                pass        # it'll error if it wasn't hooked but we don't care

    #
    # Various canned functionality
    #

    def stash_not_addr(self, addr, from_stash=None, to_stash=None):
        """
        Stash all paths not at address addr from stash from_stash to stash to_stash.
        """
        return self.stash(lambda p: p.addr != addr, from_stash=from_stash, to_stash=to_stash)

    def stash_addr(self, addr, from_stash=None, to_stash=None):
        """
        Stash all paths at address addr from stash from_stash to stash to_stash.
        """
        return self.stash(lambda p: p.addr == addr, from_stash=from_stash, to_stash=to_stash)

    def stash_addr_past(self, addr, from_stash=None, to_stash=None):
        """
        Stash all paths containg address addr in their backtrace from stash from_stash to stash to_stash.
        """
        return self.stash(lambda p: addr in p.addr_trace, from_stash=from_stash, to_stash=to_stash)

    def stash_not_addr_past(self, addr, from_stash=None, to_stash=None):
        """
        Stash all paths not containg address addr in their backtrace from stash from_stash to stash to_stash.
        """
        return self.stash(lambda p: addr not in p.addr_trace, from_stash=from_stash, to_stash=to_stash)

    def stash_all(self, from_stash=None, to_stash=None):
        """
        Stash all paths from stash from_stash to stash to_stash.
        """
        return self.stash(lambda p: True, from_stash=from_stash, to_stash=to_stash)

    def unstash_addr(self, addr, from_stash=None, to_stash=None):
        """
        Unstash all paths at address addr.
        """
        return self.unstash(lambda p: p.addr == addr, from_stash=from_stash, to_stash=to_stash)

    def unstash_addr_past(self, addr, from_stash=None, to_stash=None):
        """
        Unstash all paths containing address addr in their backtrace.
        """
        return self.unstash(lambda p: addr in p.addr_trace, from_stash=from_stash, to_stash=to_stash)

    def unstash_not_addr(self, addr, from_stash=None, to_stash=None):
        """
        Unstash all paths not at address addr.
        """
        return self.unstash(lambda p: p.addr != addr, from_stash=from_stash, to_stash=to_stash)

    def unstash_not_addr_past(self, addr, from_stash=None, to_stash=None):
        """
        Unstash all paths not containing address addr in their backtrace.
        """
        return self.unstash(lambda p: addr not in p.addr_trace, from_stash=from_stash, to_stash=to_stash)

    def unstash_all(self, from_stash=None, to_stash=None):
        """
        Unstash all paths.
        """
        return self.unstash(lambda p: True, from_stash=from_stash, to_stash=to_stash)

    #
    # High-level functionality
    #

    def explore(self, stash=None, n=None, find=None, avoid=None, find_stash='found', avoid_stash='avoid', cfg=None, num_find=1, step_func=None):
        """
        Tick stash "stash" forward (up to "n" times or until "num_find" paths are found), looking for condition "find",
        avoiding condition "avoid". Stores found paths into "find_stash' and avoided paths into "avoid_stash".

        The "find" and "avoid" parameters may be any of:

        - An address to find
        - A set or list of addresses to find
        - A function that takes a path and returns whether or not it matches.

        If an angr CFG is passed in as the "cfg" parameter and "find" is either a number or a list or a set, then
        any paths which cannot possibly reach a success state without going through a failure state will be
        preemptively avoided.
        """
        num_find += len(self.stashes[find_stash]) if find_stash in self.stashes else 0
        tech = exploration_techniques.Explorer(find=find,
                                   avoid=avoid,
                                   find_stash=find_stash,
                                   avoid_stash=avoid_stash,
                                   cfg=cfg,
                                   num_find=num_find)
        self.use_technique(tech)
        out = self.run(stash=stash,
                       step_func=step_func,
                       n=n)
        out.remove_tech(tech)
        self.remove_tech(tech)
        return out

    def run(self, stash=None, n=None, step_func=None):
        """
        Run until the path group has reached a completed state, according to
        the current exploration techniques.

        TODO: step_func doesn't work with veritesting, since veritesting replaces
        the default step logic.

        :param stash:       Operate on this stash
        :param n:           Step at most this many times
        :param step_func:   If provided, should be a function that takes a PathGroup and returns a new PathGroup. Will
                            be called with the current PathGroup at every step.
        :return:            The resulting PathGroup.
        :rtype:             PathGroup
        """
        if len(self._hooks_complete) == 0 and n is None:
            l.warn("No completion state defined for path group; stepping until all paths deadend")

        until_func = lambda pg: any(h(pg) for h in self._hooks_complete)
        return self.step(n=n, step_func=step_func, until=until_func, stash=stash)

from .path_hierarchy import PathHierarchy
from .errors import PathUnreachableError, AngrError, AngrPathGroupError
from .path import Path
from . import exploration_techniques



from ..func import Func, TestData
import random
import string

digs = string.digits + string.letters


def int2base(x, base):
    if x < 0:
        sign = -1
    elif x == 0:
        return digs[0]
    else:
        sign = 1
    x *= sign
    digits = []
    while x:
        digits.append(digs[x % base])
        x /= base
    if sign < 0:
        digits.append('-')
    digits.reverse()
    return ''.join(digits)


class strtol(Func):
    def __init__(self):
        super(strtol, self).__init__()
        self.skips_whitespace = False
        self.version = ""

    def rand_str(self, length, byte_list=None): #pylint disable=no-self-use
        if byte_list is None:
            return "".join(chr(random.randint(0, 255)) for _ in xrange(length))
        return "".join(random.choice(byte_list) for _ in xrange(length))

    def num_args(self): #pylint disable=no-self-use
        return 3

    def args(self): #pylint disable=no-self-use
        return ["nptr", "endpointer", "base"]

    def get_name(self):
        return "strtol" + self.version

    def gen_input_output_pair(self):
        num = random.randint(-(2**31), 2**31-1)

        base = random.randint(2, 16)
        s = int2base(num, base)
        test_input = [s, 0, base]
        test_output = [s, None, None]
        return_val = num
        max_steps = 20
        return TestData(test_input, test_output, return_val, max_steps)

    def pre_test(self, func, runner):
        num = random.randint(-(2 ** 31), 2 ** 31 - 1)

        s = str(num)
        test_input = [s, 0, 10]
        test_output = [s, None, None]
        return_val = num
        max_steps = 20
        test = TestData(test_input, test_output, return_val, max_steps)
        if not runner.test(func, test):
            return False

        s = str(num)
        s = self.rand_str(10, string.whitespace) + s
        test_input = [s, 0, 10]
        test_output = [s, None, None]
        return_val = num
        max_steps = 20
        test = TestData(test_input, test_output, return_val, max_steps)
        self.skips_whitespace = runner.test(func, test)

        # base 0
        base = 0
        s = hex(num)
        test_input = [s, 0, base]
        test_output = [s, None, None]
        return_val = num
        max_steps = 20
        test = TestData(test_input, test_output, return_val, max_steps)
        if not runner.test(func, test):
            return False

        return True




from simuvex.s_errors import SimError

class ExplorationTechnique(object):
    """
    An otiegnqwvk is a set of hooks for path groups that assists
    in the implementation of new techniques in symbolic exploration.

    TODO: choose actual name for the functionality (techniques? something?)

    Any number of these methods may be overridden by a subclass.
    To use an exploration technique, call ``pg.use_technique``.
    """
    # pylint: disable=unused-argument, no-self-use
    def __init__(self):
        # this attribute will be set from above by the path group
        self.project = None

    def setup(self, pg):
        """
        Perform any initialization on this path group you might need to do.
        """
        pass

    def step_path(self, path, **kwargs):
        """
        Perform the process of stepping a path forward.

        If the stepping fails, return None to fall back to a default stepping procedure.
        Otherwise, return a dict of stashes to merge into the path group. All the paths will be added to the
        PathGroup's stashes based on the mapping in the returned dict.
        """
        return None

    def step(self, pg, stash, **kwargs):
        """
        Step this stash of this path group forward.

        Return the stepped path group.
        """
        return pg.step(stash=stash, **kwargs)

    def filter(self, path):
        """
        Perform filtering on a path.

        If the path should not be filtered, return None.
        If the path should be filtered, return the name of the stash to move the path to.
        If you want to modify the path before filtering it, return a tuple of the stash to move the path to and the
        modified path.
        """
        return None

    def complete(self, pg):
        """
        Return whether or not this path group has reached a "completed" state, i.e. ``pathgroup.run()`` should halt.
        """
        return False

    def _condition_to_lambda(self, condition, default=False):
        """
        Translates an integer, set or list into a lambda that checks a path address against the given addresses, and the
        other ones from the same basic block

        :param condition:   An integer, set, or list to convert to a lambda.
        :param default:     The default return value of the lambda (in case condition is None). Default: false.

        :returns:           A lambda that takes a path and returns the set of addresses that it matched from the condition
        """
        if condition is None:
            condition_function = lambda p: default

        elif isinstance(condition, (int, long)):
            return self._condition_to_lambda((condition,))

        elif isinstance(condition, (tuple, set, list)):
            addrs = set(condition)
            def condition_function(p):
                if p.addr in addrs:
                    # returning {p.addr} instead of True to properly handle find/avoid conflicts
                    return {p.addr}

                try:
                    # If the address is not in the set (which could mean it is
                    # not at the top of a block), check directly in the blocks
                    # (Blocks are repeatedly created for every check, but with
                    # the IRSB cache in angr lifter it should be OK.)
                    return addrs.intersection(set(self.project.factory.block(p.addr).instruction_addrs))
                except (AngrError, SimError):
                    return False
        elif hasattr(condition, '__call__'):
            condition_function = condition
        else:
            raise AngrExplorationTechniqueError("ExplorationTechnique is unable to convert given type (%s) to a callable condition function." % condition.__class__)

        return condition_function

#registered_actions = {}
#registered_surveyors = {}
#
#def register_action(name, strat):
#    registered_actions[name] = strat
#
#def register_surveyor(name, strat):
#    registered_surveyors[name] = strat

from .explorer import Explorer
from .threading import Threading
from .dfs import DFS
from .looplimiter import LoopLimiter
from .lengthlimiter import LengthLimiter
from .veritesting import Veritesting
from .oppologist import Oppologist
from .director import Director, ExecuteAddressGoal, CallFunctionGoal
from .spiller import Spiller
from ..errors import AngrError, AngrExplorationTechniqueError



import simuvex
from ..analysis import register_analysis, Analysis
from ..errors import AngrValueError

import logging
l = logging.getLogger('angr.analyses.static_hooker')

class StaticHooker(Analysis):
    """
    This analysis works on statically linked binaries - it finds the library functions statically
    linked into the binary and hooks them with the appropraite simprocedures.

    Right now it only works on libc functions and unstripped binaries, but hey! There's room to
    grow!
    """

    def __init__(self):
        self.results = {}

        if self.project.loader.main_bin.linking == 'dynamic':
            raise AngrValueError('StaticHooker only works on static binaries!')

        for func in self.project.loader.main_bin._symbol_cache.values():
            if not func.is_function: continue
            try:
                self.project.hook(func.rebased_addr, simuvex.SimProcedures['libc.so.6'][func.name])
            except:     # pylint: disable=bare-except
                l.debug("Failed to hook %s at %#x", func.name, func.rebased_addr)
            else:
                l.info("Hooked %s at %#x", func.name, func.rebased_addr)
                self.results[func.rebased_addr] = simuvex.SimProcedures['libc.so.6'][func.name]

register_analysis(StaticHooker, 'StaticHooker')



import logging
import weakref
import simuvex
import claripy

l = logging.getLogger("angr.path_history")

class PathHistory(object):
    """
    This class keeps track of historically-relevant information for paths.
    """

    __slots__ = ('_parent', 'merged_from', 'merge_conditions', 'length', 'extra_length', '_addrs', '_runstr', '_target', '_guard', '_jumpkind', '_events', '_jump_source', '_jump_avoidable', '_all_constraints', '_fresh_constraints', '_satisfiable', '_state_strong_ref', '_state_weak_ref', '__weakref__')

    def __init__(self, parent=None):
        self._parent = parent
        self.merged_from = [ ]
        self.merge_conditions = [ ]
        self._runstr = None
        self._target = None
        self._jump_source = None
        self._jump_avoidable = None
        self._guard = None
        self._jumpkind = None
        self._events = None
        self._addrs = ()

        self.length = 0 if parent is None else parent.length + 1
        self.extra_length = 0 if parent is None else parent.extra_length

        # satness stuff
        self._all_constraints = ()
        self._fresh_constraints = ()
        self._satisfiable = None

        # the state itself
        self._state_weak_ref = None
        self._state_strong_ref = None

    def copy(self):
        c = PathHistory()
        c._parent = self._parent
        c.merged_from = list(self.merged_from)
        c.merge_conditions = list(self.merge_conditions)
        c._runstr = self._runstr
        c._target = self._target
        c._jump_source = self._jump_source
        c._jump_avoidable = self._jump_avoidable
        c._guard = self._guard
        c._jumpkind = self._jumpkind
        c._events = self._events
        c._addrs = self._addrs

        c.length = self.length
        c.extra_length = self.extra_length

        c._all_constraints = list(self._all_constraints)
        c._fresh_constraints = list(self._fresh_constraints)
        c._satisfiable = self._satisfiable
        c._state_weak_ref = self._state_weak_ref
        c._state_strong_ref = self._state_strong_ref

        return c

    def __getstate__(self):
        return [
            (k, getattr(self, k)) for k in self.__slots__ if k not in
            ('__weakref__', '_state_weak_ref')
        ]

    def __setstate__(self, state):
        for k,v in state:
            setattr(self, k, v)
        self._state_weak_ref = None

    def _record_state(self, state, strong_reference=True):
        self._jumpkind = state.scratch.jumpkind
        self._jump_source = state.scratch.source
        self._jump_avoidable = state.scratch.avoidable
        self._target = state.scratch.target
        self._guard = state.scratch.guard

        if state.scratch.bbl_addr_list is not None:
            self._addrs = state.scratch.bbl_addr_list
        elif state.scratch.bbl_addr is not None:
            self._addrs = [ state.scratch.bbl_addr ]
        else:
            # state.scratch.bbl_addr may not be initialized as final states from the "flat_successors" list. We need to get
            # the value from _target in that case.
            if self.addr is None and not self._target.symbolic:
                self._addrs = [ self._target._model_concrete.value ]
            else:
                # FIXME: redesign so this does not happen
                l.warning("Encountered a path to a SimProcedure with a symbolic target address.")

        if simuvex.o.UNICORN in state.options:
            self.extra_length += state.scratch.executed_block_count - 1

        if simuvex.o.TRACK_ACTION_HISTORY in state.options:
            self._events = state.log.events

        # record constraints, added constraints, and satisfiability
        self._all_constraints = state.se.constraints
        self._fresh_constraints = state.log.fresh_constraints

        if isinstance(state.se._solver, claripy.frontend_mixins.SatCacheMixin):
            self._satisfiable = state.se._solver._cached_satness
        else:
            self._satisfiable = None

        # record the state as a weak reference
        self._state_weak_ref = weakref.ref(state)

        # and as a strong ref
        if strong_reference:
            self._state_strong_ref = state

    def _record_run(self, run):
        self._runstr = str(run)

    @property
    def state(self):
        return (
            self._state_strong_ref if self._state_strong_ref is not None else
            self._state_weak_ref() if self._state_weak_ref is not None else
            None
        )

    def demote(self):
        """
        Demotes this PathHistory node, causing it to convert references to the state
        to weakrefs.
        """
        self._state_strong_ref = None

    #
    # Some GC-dependent pass-throughts to the state
    #

    @property
    def events(self):
        if self._events is not None:
            return self._events
        elif self.state is not None:
            return self.state.log.events
        else:
            return ()

    def reachable(self):
        if self._satisfiable is not None:
            pass
        elif self.state is not None:
            self._satisfiable = self.state.se.satisfiable()
        else:
            solver = claripy.Solver()
            solver.add(self._all_constraints)
            self._satisfiable = solver.satisfiable()

        return self._satisfiable

    @property
    def actions(self):
        return [ ev for ev in self.events if isinstance(ev, simuvex.SimAction) ]

    @property
    def addr(self):
        return self._addrs[0]

    @addr.setter
    def addr(self, v):
        self._addrs = [ v ]

    def closest_common_ancestor(self, other):
        """
        Find the common ancestor between this PathHistory and 'other'.

        :param other:    the PathHistory to find a common ancestor with.
        :return:        the common ancestor PathHistory, or None if there isn't one
        """
        our_history_iter = reversed(HistoryIter(self))
        their_history_iter = reversed(HistoryIter(other))
        sofar = set()

        while True:
            our_done = False
            their_done = False

            try:
                our_next = next(our_history_iter)
                if our_next in sofar:
                    # we found it!
                    return our_next
                sofar.add(our_next)
            except StopIteration:
                # we ran out of items during iteration
                our_done = True

            try:
                their_next = next(their_history_iter)
                if their_next in sofar:
                    # we found it!
                    return their_next
                sofar.add(their_next)
            except StopIteration:
                # we ran out of items during iteration
                their_done = True

            # if we ran out of both lists, there's no common ancestor
            if our_done and their_done:
                return None

    def constraints_since(self, other):
        """
        Returns the constraints that have been accumulated since `other`.

        :param other: a prior PathHistory object
        :returns: a list of constraints
        """

        constraints = [ ]
        cur = self
        while cur is not other and cur is not None:
            constraints.extend(cur._fresh_constraints)
            cur = cur._parent
        return constraints

class TreeIter(object):
    def __init__(self, start, end=None):
        self._start = start
        self._end = end

    def _iter_nodes(self):
        n = self._start
        while n is not self._end:
            yield n
            n = n._parent

    def __iter__(self):
        for i in self.hardcopy:
            yield i

    def __reversed__(self):
        raise NotImplementedError("Why are you using this class")

    @property
    def hardcopy(self):
        # lmao
        return list(reversed(tuple(reversed(self))))

    def __len__(self):
        return self._start.length

    def __getitem__(self, k):
        if isinstance(k, slice):
            raise ValueError("Please use .hardcopy to use slices")
        if k >= 0:
            raise ValueError("Please use .hardcopy to use nonnegative indexes")
        i = 0
        for item in reversed(self):
            i -= 1
            if i == k:
                return item
        raise IndexError(k)

    def count(self, v):
        """
        Count occurrences of value v in the entire history. Note that the subclass must implement the __reversed__
        method, otherwise an exception will be thrown.
        :param object v: The value to look for
        :return: The number of occurrences
        :rtype: int
        """
        ctr = 0
        for item in reversed(self):
            if item == v:
                ctr += 1
        return ctr

class HistoryIter(TreeIter):
    def __reversed__(self):
        for hist in self._iter_nodes():
            yield hist

class AddrIter(TreeIter):
    def __reversed__(self):
        for hist in self._iter_nodes():
            for a in reversed(hist._addrs):
                yield a

class RunstrIter(TreeIter):
    def __reversed__(self):
        for hist in self._iter_nodes():
            if hist._runstr is not None:
                yield hist._runstr

class TargetIter(TreeIter):
    def __reversed__(self):
        for hist in self._iter_nodes():
            if hist._target is not None:
                yield hist._target

class GuardIter(TreeIter):
    def __reversed__(self):
        for hist in self._iter_nodes():
            if hist._guard is not None:
                yield hist._guard

class JumpkindIter(TreeIter):
    def __reversed__(self):
        for hist in self._iter_nodes():
            if hist._jumpkind is not None:
                yield hist._jumpkind

class EventIter(TreeIter):
    def __reversed__(self):
        for hist in self._iter_nodes():
            for ev in reversed(hist.events):
                yield ev

class ActionIter(TreeIter):
    def __reversed__(self):
        for hist in self._iter_nodes():
            for ev in reversed(hist.actions):
                yield ev



from ..func import Func, TestData
import random
import string


class atoi(Func):
    def __init__(self):
        super(atoi, self).__init__()
        self.skips_whitespace = False
        self.allows_negative = True

    def rand_str(self, length, byte_list=None): #pylint disable=no-self-use
        if byte_list is None:
            return "".join(chr(random.randint(0, 255)) for _ in xrange(length))
        return "".join(random.choice(byte_list) for _ in xrange(length))

    def num_args(self):
        return 1

    def get_name(self):
        if self.allows_negative:
            suffix = ""
        else:
            suffix = "_no_signs"
        if self.skips_whitespace:
            return "atoi_whitespace_skip" + suffix
        return "atoi" + suffix

    def gen_input_output_pair(self):
        num = random.randint(-(2**26), 2**26-1)

        if not self.allows_negative:
            num = abs(num)

        s = str(num)
        test_input = [s]
        test_output = [s]
        return_val = num
        max_steps = 20
        return TestData(test_input, test_output, return_val, max_steps)

    def pre_test(self, func, runner):
        num = random.randint(-(2 ** 26), 2 ** 26 - 1)

        num = abs(num)
        s = str(num)
        test_input = [s]
        test_output = [s]
        return_val = num
        max_steps = 20
        test = TestData(test_input, test_output, return_val, max_steps)
        if not runner.test(func, test):
            return False

        s = str(num)
        s = self.rand_str(10, string.whitespace) + s
        test_input = [s]
        test_output = [s]
        return_val = num
        max_steps = 20
        test = TestData(test_input, test_output, return_val, max_steps)
        self.skips_whitespace = runner.test(func, test)

        num = -random.randint(2000, 8000)
        s = str(num)
        test_input = [s]
        test_output = [s]
        return_val = num
        max_steps = 20
        test = TestData(test_input, test_output, return_val, max_steps)
        if not runner.test(func, test):
            self.allows_negative = False

        return True



from . import ExplorationTechnique

class LoopLimiter(ExplorationTechnique):
    """
    Limit the number of loops a path may go through.
    Paths that exceed the loop limit are moved to a discard stash.

    Note that this uses the default detect_loops method from Path, which approximates loop
    counts by counting the number of times each basic block is executed in a given stack frame.
    """
    def __init__(self, count=5, discard_stash='spinning'):
        super(LoopLimiter, self).__init__()
        self.count = count
        self.discard_stash = discard_stash

    def step(self, pg, stash, **kwargs):
        pg = pg.step(stash=stash, **kwargs).move(stash, self.discard_stash,
                lambda path: path.detect_loops() >= self.count)
        if len(pg.stashes[stash]) == 0 and len(pg.stashes[self.discard_stash]) > 0:
            pg.stashes[stash].append(pg.stashes[self.discard_stash].pop())
        return pg

